/* ************************************************************************
 *   File: act.movement.c                                Part of CircleMUD *
 *  Usage: movement commands, door handling, & sleep/rest/etc state        *
 *                                                                         *
 *  All rights reserved.  See license.doc for complete information.        *
 *                                                                         *
 *  Copyright (C) 1993, 94 by the Trustees of the Johns Hopkins University *
 *  CircleMUD is based on DikuMUD, Copyright (C) 1990, 1991.               *
 ************************************************************************ */

#include "conf.h"
#include "sysdep.h"

#include "structs.h"
#include "utils.h"

#include "coins.h"
#include "comm.h"
#include "constants.h"
#include "db.h"
#include "handler.h"
#include "house.h"
#include "interpreter.h"
#include "mobprog.h"
#include "screen.h"
#include "spells.h"


/* external variables  */
extern int tunnel_size;
extern int allow_recall;
extern room_rnum r_mortal_start_room;
extern room_rnum r_immort_start_room;
extern room_rnum r_frozen_start_room;
extern int num_of_houses;
extern struct house_control_rec house_control[MAX_HOUSES];

/* external functions */
int special(struct char_data *ch, int cmd, char *arg);
void death_cry(struct char_data *ch);
int find_eq_pos(struct char_data *ch, struct obj_data *obj, char *arg);
int in_water(struct char_data *ch);
int can_recall(struct char_data *ch); 

/* local functions */
int has_boat(struct char_data *ch);
int find_door(struct char_data *ch, const char *type, char *dir, const char *cmdname);
int has_key(struct char_data *ch, obj_vnum key);
void do_doorcmd(struct char_data *ch, struct obj_data *obj, int door, int scmd);
int ok_pick(struct char_data *ch, obj_vnum keynum, int pickproof, int scmd);
void dismount_char(struct char_data * ch);
void mount_char(struct char_data *ch, struct char_data *mount);

ACMD(do_gen_door);
ACMD(do_enter);
ACMD(do_leave);
ACMD(do_stand);
ACMD(do_sit);
ACMD(do_recall);
ACMD(do_rest);
ACMD(do_sleep);
ACMD(do_wake);
ACMD(do_follow);


/* simple function to determine if char can walk on water */
int has_boat(struct char_data *ch)
{
	struct obj_data *obj;
	int i;

/*
  if (ROOM_IDENTITY(IN_ROOM(ch)) == DEAD_SEA)
  return (1);
*/

	if (GET_LEVEL(ch) > LVL_IMMORT)
		return (1);

	if (AFF_FLAGGED(ch, AFF_WATERWALK))
		return (1);

	/* non-wearable boats in inventory will do it */
	for (obj = ch->carrying; obj; obj = obj->next_content)
		if (GET_OBJ_TYPE(obj) == ITEM_BOAT && (find_eq_pos(ch, obj, NULL) < 0))
			return (1);

	/* and any boat you're wearing will do it too */
	for (i = 0; i < NUM_WEARS; i++)
		if (GET_EQ(ch, i) && GET_OBJ_TYPE(GET_EQ(ch, i)) == ITEM_BOAT)
			return (1);

	/* flying isn't really a boat, but it accomplishes the same goal */
	if (GET_POS(ch) == POS_FLYING)
		return (1);

	return (0);
}

  

/* do_simple_move assumes
 *    1. That there is no master and no followers.
 *    2. That the direction exists.
 *
 *   Returns :
 *   1 : If succes.
 *   0 : If fail
 */
int do_simple_move(struct char_data *ch, int dir, int need_specials_check)
{
	char throwaway[MAX_INPUT_LENGTH] = ""; /* Functions assume writable. */
	room_rnum was_in;
	int need_movement;
	char buf2[MAX_STRING_LENGTH];

	int goal, roll;
   int same_room = 0, riding = 0, ridden_by = 0;
   room_vnum vnum;

	/*
	 * Check for special routines (North is 1 in command list, but 0 here) Note
	 * -- only check if following; this avoids 'double spec-proc' bug
	 */
	if (need_specials_check && special(ch, dir + 1, throwaway))
		return (0);

   /* check if they're mounted */
   if (RIDING(ch))
		riding = TRUE;
   if (RIDDEN_BY(ch))
		ridden_by = TRUE;
   
   /* if they're mounted, are they in the same room w/ their mount(ee)? */
   if (riding && IN_ROOM(RIDING(ch)) == IN_ROOM(ch))
		same_room = TRUE;
   else if (ridden_by && IN_ROOM(RIDDEN_BY(ch)) == IN_ROOM(ch))
		same_room = TRUE;
 
   /* tamed mobiles cannot move about (DAK) */
   if (ridden_by && same_room && AFF_FLAGGED(ch, AFF_TAMED)) {
		send_to_char(ch, "You've been tamed.  Now act it!\r\n");
		return (0);
   }
 
	/* charmed? */
	if (AFF_FLAGGED(ch, AFF_CHARM) && ch->master && IN_ROOM(ch) == IN_ROOM(ch->master)) {
		send_to_char(ch, "The thought of leaving your master makes you weep.\r\n");
		act("$n bursts into tears.", FALSE, ch, 0, 0, TO_ROOM);
		return (0);
	}

	/* if this room or the one we're going to needs a boat, check for one */
	if ((SECT(IN_ROOM(ch)) == SECT_WATER_NOSWIM) ||
		 (SECT(EXIT(ch, dir)->to_room) == SECT_WATER_NOSWIM)) {
		if ((riding && !has_boat(RIDING(ch))) || !has_boat(ch)) {
			send_to_char(ch, "You need a boat to go there.\r\n");
			return (0);
		}
	}

	/* move points needed is avg. move loss for src and destination sect type */
	need_movement = (movement_loss[SECT(IN_ROOM(ch))] +
						  movement_loss[SECT(EXIT(ch, dir)->to_room)]) / 2;

	/* Bad weather impedes movement */
	if (OUTSIDE(ch)) {
		if (zone_table[IN_ZONE(ch)].sky == SKY_RAINING)
			need_movement *= 2;
		else if (zone_table[IN_ZONE(ch)].sky == SKY_LIGHTNING)
			need_movement *= 3;
	}
	
	if (GET_POS(ch) == POS_FLYING) {
		need_movement /= 2;

		if (need_movement < 1)
			need_movement = 1;
	}
  
   if (riding) {
		if (GET_MOVE(RIDING(ch)) < need_movement) {
			send_to_char(ch, "Your mount is too exhausted.\r\n");
			return (0);
		}
   } else {
		if (GET_MOVE(ch) < need_movement && !IS_NPC(ch)) {
			if (need_specials_check && ch->master)
				send_to_char(ch, "You are too exhausted to follow.\r\n");
			else
				send_to_char(ch, "You are too exhausted.\r\n");
			return (0);
		}
   }
   
	goal = (IS_NPC(ch) ? 25 : GET_SKILL(ch, SKILL_RIDING));
	roll = rand_number(1, 101) - rand_number(-4, need_movement);
	
   if (riding && roll > goal) {
		act("$N rears backwards, throwing you to the ground.", 
			 FALSE, ch, 0, RIDING(ch), TO_CHAR);
		act("You rear backwards, throwing $n to the ground.", 
			 FALSE, ch, 0, RIDING(ch), TO_VICT);
		act("$N rears backwards, throwing $n to the ground.", 
			 FALSE, ch, 0, RIDING(ch), TO_NOTVICT);
		dismount_char(ch);
		damage(ch, ch, dice(1,6), -1);
		return (0);
	}

   vnum = GET_ROOM_VNUM(EXIT(ch, dir)->to_room);

	if (ROOM_FLAGGED(IN_ROOM(ch), ROOM_ATRIUM)) {
		if (!House_can_enter(ch, vnum)) {
			send_to_char(ch, "That's private property -- no trespassing!\r\n");
			return (0);
		}
	}

	if (ROOM_FLAGGED(EXIT(ch, dir)->to_room, ROOM_TUNNEL)) {
		if (riding || ridden_by) {
			send_to_char(ch, "There isn't enough room there, while mounted.\r\n");
			return (0);
		} else {
			if (num_pc_in_room(&(world[EXIT(ch, dir)->to_room])) >= tunnel_size) {
				if (tunnel_size > 1)
					send_to_char(ch, "There isn't enough room for you to go there!\r\n");
				else
					send_to_char(ch, "There isn't enough room there for more "
									 "than one person!\r\n");
				return (0);
			}
		}
	}
		 
	/* Mortals and low level gods cannot enter greater god rooms. */
	if (ROOM_FLAGGED(EXIT(ch, dir)->to_room, ROOM_GODROOM) &&
		 GET_LEVEL(ch) < LVL_GRGOD) {
		send_to_char(ch, "You aren't godly enough to use that room!\r\n");
		return (0);
	}

	if (GET_LEVEL(ch) < LVL_IMMORT) {
		/* So classes can't enter no_class rooms */
		if (ROOM_FLAGGED(EXIT(ch, dir)->to_room, ROOM_NO_MAGE) &&
			 IS_MAGIC_USER(ch)) {
			send_to_char(ch, "An eldritch force blocks your passage.\r\n");
			return (0);
		} 
		if (ROOM_FLAGGED(EXIT(ch, dir)->to_room, ROOM_NO_CLERIC) &&
			 IS_CLERIC(ch)) {
			send_to_char(ch, "Your god prevents you from going that way!\r\n");
			return (0);
		} 
		if (ROOM_FLAGGED(EXIT(ch, dir)->to_room, ROOM_NO_THIEF) && 
			 IS_THIEF(ch)) {
			send_to_char(ch, "No thieves allowed!  Shoo!\r\n");
			return (0);
		} 
		if (ROOM_FLAGGED(EXIT(ch, dir)->to_room, ROOM_NO_WARRIOR) &&
			 IS_WARRIOR(ch)) {
			send_to_char(ch, "No warriors allowed, go away!\r\n");
			return (0);
		} 
	}

	/* Now we know we're allow to go into the room. */
	if (GET_LEVEL(ch) < LVL_IMMORT && !IS_NPC(ch) && !(riding || ridden_by))
		GET_MOVE(ch) -= need_movement;
   else if (riding)
		GET_MOVE(RIDING(ch)) -= need_movement;
   else if (ridden_by)
		GET_MOVE(RIDDEN_BY(ch)) -= need_movement;
 
   if (riding) {
		if (GET_POS(RIDING(ch)) == POS_FLYING) {
			snprintf(buf2, sizeof(buf2), 
						"$n flies away on $N to the %s.", dirs[dir]);
			act(buf2, TRUE, ch, 0, RIDING(ch), TO_ROOM);
		}
		else if (!AFF_FLAGGED(RIDING(ch), AFF_SNEAK)) {
			if (AFF_FLAGGED(ch, AFF_SNEAK)) {
				snprintf(buf2, sizeof(buf2), "$n leaves %s.", dirs[dir]);
				act(buf2, TRUE, RIDING(ch), 0, 0, TO_ROOM);
			} else {
				snprintf(buf2, sizeof(buf2), "$n rides $N %s.", dirs[dir]);
				act(buf2, TRUE, ch, 0, RIDING(ch), TO_NOTVICT);
			}
		}
	} else if (ridden_by) {
		if (GET_POS(ch) == POS_FLYING) {
			snprintf(buf2, sizeof(buf2), 
						"$n flies away, carrying $N to the %s.", dirs[dir]);
			act(buf2, TRUE, ch, 0, RIDDEN_BY(ch), TO_ROOM);
		} else if (!AFF_FLAGGED(ch, AFF_SNEAK)) {
			if (AFF_FLAGGED(RIDDEN_BY(ch), AFF_SNEAK)) {
				snprintf(buf2, sizeof(buf2), "$n leaves %s.", dirs[dir]);
				act(buf2, TRUE, ch, 0, 0, TO_ROOM);
			} else {
				snprintf(buf2, sizeof(buf2), "$n rides $N %s.", dirs[dir]);
				act(buf2, TRUE, RIDDEN_BY(ch), 0, ch, TO_NOTVICT);
			}
		} 
   } else if (!AFF_FLAGGED(ch, AFF_SNEAK)) {
		if (GET_POS(ch) == POS_FLYING)
			snprintf(buf2, sizeof(buf2), "$n flies away to the %s.", dirs[dir]);
		else if (!AFF_FLAGGED(ch, AFF_SNEAK))
			snprintf(buf2, sizeof(buf2), "$n leaves %s.", dirs[dir]);
		else
			buf2[0] = '\0';

		if (*buf2)
			act(buf2, TRUE, ch, 0, 0, TO_ROOM);
	}
	
	was_in = IN_ROOM(ch);
	if (in_water(ch) && 
		 (PLR_FLAGGED(ch, PLR_FISHING) || PLR_FLAGGED(ch, PLR_FISH_ON))) {
		PLR_FLAGS(ch).PLR_FISHING = 0;
		PLR_FLAGS(ch).PLR_FISH_ON = 0;
		send_to_char(ch, "\r\nYou pack up your fishing gear and move on.\r\n");
	}

	char_from_room(ch);
	char_to_room(ch, world[was_in].dir_option[dir]->to_room);

   if (riding && same_room && IN_ROOM(RIDING(ch)) != IN_ROOM(ch)) {
		char_from_room(RIDING(ch));
		char_to_room(RIDING(ch), IN_ROOM(ch));
   } else if (ridden_by && same_room && IN_ROOM(RIDDEN_BY(ch)) != IN_ROOM(ch)){
		char_from_room(RIDDEN_BY(ch));
		char_to_room(RIDDEN_BY(ch), IN_ROOM(ch));
	}
	
   if (!AFF_FLAGGED(ch, AFF_SNEAK)) {
		if (riding && same_room && !AFF_FLAGGED(RIDING(ch), AFF_SNEAK)) {
			if (GET_POS(RIDING(ch)) == POS_FLYING)
				act("$n flies in on $N.", TRUE, ch, 0, RIDING(ch), TO_ROOM);
			else if (!AFF_FLAGGED(ch, AFF_SNEAK))
				act("$n arrives, riding $N.", TRUE, ch, 0, RIDING(ch), TO_ROOM);
		} else if (ridden_by && same_room && !AFF_FLAGGED(RIDDEN_BY(ch), AFF_SNEAK)) {
			if (GET_POS(ch) == POS_FLYING)
				act("$n flies in on $N.", TRUE, RIDDEN_BY(ch), 0, ch, TO_ROOM);
			else if (!AFF_FLAGGED(ch, AFF_SNEAK))
				act("$n arrives, riding $N.", TRUE, RIDDEN_BY(ch), 0, ch, TO_ROOM);
		} else if (!riding || (riding && !same_room)) {
			if (GET_POS(ch) == POS_FLYING)
				act("$n flies in.", TRUE, ch, 0, 0, TO_ROOM);
			else if (!AFF_FLAGGED(ch, AFF_SNEAK))
				act("$n has arrived.", TRUE, ch, 0, 0, TO_ROOM);
		}
	}
	
	if (ch->desc != NULL)
		look_at_room(ch, 0);

	/* DT! (Hopefully these are rare in your MUD) -dak */
	if (ROOM_FLAGGED(IN_ROOM(ch), ROOM_DEATH)) {
		if (riding && GET_LEVEL(RIDING(ch)) < LVL_IMMORT) {
			log_death_trap(RIDING(ch));
			death_cry(RIDING(ch));
			extract_char(RIDING(ch));
		}
     
		if (ridden_by && GET_LEVEL(RIDDEN_BY(ch)) < LVL_IMMORT) {
			log_death_trap(RIDDEN_BY(ch));
			death_cry(RIDDEN_BY(ch));
			extract_char(RIDDEN_BY(ch));
		}
		
		if (GET_LEVEL(ch) < LVL_IMMORT) {
			log_death_trap(ch);
			death_cry(ch);
			extract_char(ch);
		}
		return (0);
	}
	
	mprog_entry_trigger(ch);
	mprog_greet_trigger(ch);
	return (1);
}


int perform_move(struct char_data *ch, int dir, int need_specials_check)
{
	room_rnum was_in;
	struct follow_type *k, *next;

	if (ch == NULL || dir < 0 || dir >= NUM_OF_DIRS || FIGHTING(ch))
		return (0);
	else if (!EXIT(ch, dir) || EXIT(ch, dir)->to_room == NOWHERE)
		send_to_char(ch, "Alas, you cannot go that way...\r\n");

	/* Don't reveal the presence of hidden doors! */
	else if (EXIT_FLAGGED(EXIT(ch, dir), EX_HIDDEN) &&
				EXIT_FLAGGED(EXIT(ch, dir), EX_CLOSED))
		send_to_char(ch, "Alas, you cannot go that way...\r\n");

	else if (EXIT_FLAGGED(EXIT(ch, dir), EX_CLOSED)) {
		if (EXIT(ch, dir)->keyword)
			send_to_char(ch, "The %s seems to be closed.\r\n", fname(EXIT(ch, dir)->keyword));
		else
			send_to_char(ch, "It seems to be closed.\r\n");
	} else {
		if (!ch->followers)
			return (do_simple_move(ch, dir, need_specials_check));

		was_in = IN_ROOM(ch);
		if (!do_simple_move(ch, dir, need_specials_check))
			return (0);

		for (k = ch->followers; k; k = next) {
			next = k->next;
			if ((IN_ROOM(k->follower) == was_in) &&
				 (GET_POS(k->follower) >= POS_STANDING)) {
				act("You follow $N.\r\n", FALSE, k->follower, 0, ch, TO_CHAR);
				perform_move(k->follower, dir, 1);
			}
		}
		return (1);
	}
	return (0);
}


ACMD(do_speedwalk) {
  int dir, r;

  for (r = 1; *argument && r; argument++) {
    while (*argument == ' ')
      ++argument;
    
    switch (*argument) {
      case 'N':
      case 'n':
        dir = NORTH;
        break;
      case 'E':
      case 'e':
        dir = EAST;
        break;
      case 'S':
      case 's':
        dir = SOUTH;
        break;
      case 'W':
      case 'w':
        dir = WEST;
        break;
      case 'U':
      case 'u':
        dir = UP;
        break;
      case 'D':
      case 'd':
        dir = DOWN;
        break;
      default:
        send_to_char(ch, "Alas, you can't go that way.\r\n");
        return;
    }    
    
    r = perform_move(ch, dir, 1);
    if (r && *(argument + 1))
      send_to_char(ch, "\r\n");
  }
}


ACMD(do_move)
{
	register struct char_data *vict, *tmob;
	int found;

	/*
	 * This is basically a mapping of cmd numbers to perform_move indices.
	 * It cannot be done in perform_move because perform_move is called
	 * by other functions which do not require the remapping.
	 */
	perform_move(ch, subcmd - 1, 0);

	/* LJ Fast Aggressive */
	for (tmob = world[IN_ROOM(ch)].people; tmob; tmob = tmob->next_in_room)
	{
		if (!IS_NPC(tmob) || !MOB_FLAGGED(tmob, MOB_HYPER_AGGR))
			continue;

		found = FALSE;
		for (vict = world[IN_ROOM(tmob)].people; 
			  vict && !found; 
			  vict = vict->next_in_room)
		{
			if (IS_NPC(vict) ||
				 !CAN_SEE(tmob, vict) || 
				 PRF_FLAGGED(vict, PRF_NOHASSLE))
            continue;

			if (MOB_FLAGGED(tmob, MOB_WIMPY) && AWAKE(vict))
            continue;

			if (MOB_FLAGGED(tmob, MOB_AGGRESSIVE) ||
				 (MOB_FLAGGED(tmob, MOB_AGGR_EVIL) && IS_EVIL(vict)) ||
				 (MOB_FLAGGED(tmob, MOB_AGGR_NEUTRAL) && IS_NEUTRAL(vict)) ||
				 (MOB_FLAGGED(tmob, MOB_AGGR_GOOD) && IS_GOOD(vict))) {
            hit(tmob, vict, TYPE_UNDEFINED);
            found = TRUE;
			}
		}
	}
}


int find_door(struct char_data *ch, const char *type, char *dir, const char *cmdname)
{
	int door;

	if (*dir) {			/* a direction was specified */
		if ((door = search_block(dir, dirs, FALSE)) == -1) {	/* Partial Match */
			send_to_char(ch, "That's not a direction.\r\n");
			return (-1);
		}
		if (EXIT(ch, door) && !EXIT_FLAGGED(EXIT(ch, door), EX_HIDDEN)) {
			if (EXIT(ch, door)->keyword) {
				if (isname(type, EXIT(ch, door)->keyword))
					return (door);
				else {
					send_to_char(ch, "I see no %s there.\r\n", type);
					return (-1);
				}
			} else
				return (door);
		} else {
			send_to_char(ch, "I really don't see how you can %s anything "
							 "there.\r\n", cmdname);
			return (-1);
		}
	} else {			/* try to locate the keyword */
		if (!*type) {
			send_to_char(ch, "What is it you want to %s?\r\n", cmdname);
			return (-1);
		}
		for (door = 0; door < NUM_OF_DIRS; door++)
			if (EXIT(ch, door) && !EXIT_FLAGGED(EXIT(ch, door), EX_HIDDEN))
				if (EXIT(ch, door)->keyword)
					if (isname(type, EXIT(ch, door)->keyword))
						return (door);

		send_to_char(ch, "There doesn't seem to be %s %s here.\r\n", AN(type), type);
		return (-1);
	}
}


int has_key(struct char_data *ch, obj_vnum key)
{
	struct obj_data *o;

	for (o = ch->carrying; o; o = o->next_content)
		if (GET_OBJ_VNUM(o) == key)
			return (1);

	if (GET_EQ(ch, WEAR_HOLD))
		if (GET_OBJ_VNUM(GET_EQ(ch, WEAR_HOLD)) == key)
			return (1);

	return (0);
}


struct needs_to_be {
	BIT(NEED_OPEN);
	BIT(NEED_CLOSED);
	BIT(NEED_UNLOCKED);
	BIT(NEED_LOCKED);
};

const struct needs_to_be flags_door[] = {
	{ 0, 1, 1, 0 },       /* closed, unlocked     */
	{ 1, 0, 0, 0 },       /* open                 */
	{ 0, 1, 0, 1 },       /* closed, locked       */
	{ 0, 1, 1, 0 },       /* closed, unlocked     */
	{ 0, 1, 0, 1 }        /* closed, locked       */
};

const char *cmd_door[] =
{
	"open",
	"close",
	"unlock",
	"lock",
	"pick"
};

#define EXITN(room, door)		(world[room].dir_option[door])
#define OPEN_DOOR(room, obj, door)	((obj) ?\
		(REMOVE_BIT(GET_OBJ_VAL(obj, 1), CONT_CLOSED)) :\
		(EXITN(room, door)->exit_info.EX_CLOSED = 0))
#define CLOSE_DOOR(room, obj, door)	((obj) ?\
		(SET_BIT(GET_OBJ_VAL(obj, 1), CONT_CLOSED)) :\
		(EXITN(room, door)->exit_info.EX_CLOSED = 1))
#define LOCK_DOOR(room, obj, door)	((obj) ?\
		(SET_BIT(GET_OBJ_VAL(obj, 1), CONT_LOCKED)) :\
		(EXITN(room, door)->exit_info.EX_LOCKED = 1))
#define UNLOCK_DOOR(room, obj, door)	((obj) ?\
		(REMOVE_BIT(GET_OBJ_VAL(obj, 1), CONT_LOCKED)) :\
		(EXITN(room, door)->exit_info.EX_LOCKED = 0))
#define TOGGLE_LOCK(room, obj, door)	((obj) ?\
		(TOGGLE_BIT(GET_OBJ_VAL(obj, 1), CONT_LOCKED)) :\
		(TOGGLE_FIELD(EXITN(room, door)->exit_info, EX_LOCKED)))

void do_doorcmd(struct char_data *ch, struct obj_data *obj, int door, int scmd)
{
	char buf[MAX_STRING_LENGTH];
	size_t len;
	room_rnum other_room = NOWHERE;
	struct room_direction_data *back = NULL;

	len = snprintf(buf, sizeof(buf), "$n %ss ", cmd_door[scmd]);
	if (!obj && ((other_room = EXIT(ch, door)->to_room) != NOWHERE))
		if ((back = world[other_room].dir_option[rev_dir[door]]) != NULL)
			if (back->to_room != IN_ROOM(ch))
				back = NULL;

	switch (scmd) {
		case SCMD_OPEN:
			OPEN_DOOR(IN_ROOM(ch), obj, door);
			if (back)
				OPEN_DOOR(other_room, obj, rev_dir[door]);
			send_to_char(ch, "%s", OK);
			break;

		case SCMD_CLOSE:
			CLOSE_DOOR(IN_ROOM(ch), obj, door);
			if (back)
				CLOSE_DOOR(other_room, obj, rev_dir[door]);
			send_to_char(ch, "%s", OK);
			break;

		case SCMD_LOCK:
			LOCK_DOOR(IN_ROOM(ch), obj, door);
			if (back)
				LOCK_DOOR(other_room, obj, rev_dir[door]);
			send_to_char(ch, "*Click*\r\n");
			break;

		case SCMD_UNLOCK:
			UNLOCK_DOOR(IN_ROOM(ch), obj, door);
			if (back)
				UNLOCK_DOOR(other_room, obj, rev_dir[door]);
			send_to_char(ch, "*Click*\r\n");
			break;

		case SCMD_PICK:
			TOGGLE_LOCK(IN_ROOM(ch), obj, door);
			if (back)
				TOGGLE_LOCK(other_room, obj, rev_dir[door]);
			send_to_char(ch, "The lock quickly yields to your skills.\r\n");
			len = strlcpy(buf, "$n skillfully picks the lock on ", sizeof(buf));
			break;
	}

	/* Notify the room. */
	if (len < sizeof(buf))
		snprintf(buf + len, sizeof(buf) - len, "%s%s.",
					obj ? "" : "the ", obj ? "$p" : EXIT(ch, door)->keyword ? "$F" : "door");
	if (!obj || IN_ROOM(obj) != NOWHERE)
		act(buf, FALSE, ch, obj, obj ? 0 : EXIT(ch, door)->keyword, TO_ROOM);

	/* Notify the other room */
	if (back && (scmd == SCMD_OPEN || scmd == SCMD_CLOSE))
      send_to_room(EXIT(ch, door)->to_room, "The %s is %s%s from the other side.",
						 back->keyword ? fname(back->keyword) : "door", cmd_door[scmd],
						 scmd == SCMD_CLOSE ? "d" : "ed");
}


int ok_pick(struct char_data *ch, obj_vnum keynum, int pickproof, int scmd)
{
	int roll = rand_number(1, 101);
	int goal = (!IS_NPC(ch) ? GET_SKILL(ch, SKILL_PICK_LOCK) : 0);
	if (goal) {
		goal += dex_app_skill[GET_DEX(ch)].p_locks;
		goal += race_app_skill[GET_RACE(ch)].p_locks;
	}
	
	if (scmd == SCMD_PICK) {
		if (keynum == NOTHING)
			send_to_char(ch, "Odd - you can't seem to find a keyhole.\r\n");
		else if (pickproof)
			send_to_char(ch, "It resists your attempts to pick it.\r\n");
		else if (roll > goal)
			send_to_char(ch, "You failed to pick the lock.\r\n");
		else
			return (1);
		return (0);
	}
	return (1);
}


#define DOOR_IS_OPENABLE(ch, obj, door)	((obj) ? \
			((GET_OBJ_TYPE(obj) == ITEM_CONTAINER) && \
			OBJVAL_FLAGGED(obj, CONT_CLOSEABLE)) :\
			(EXIT_FLAGGED(EXIT(ch, door), EX_ISDOOR)))
#define DOOR_IS_OPEN(ch, obj, door)	((obj) ? \
			(!OBJVAL_FLAGGED(obj, CONT_CLOSED)) :\
			(!EXIT_FLAGGED(EXIT(ch, door), EX_CLOSED)))
#define DOOR_IS_UNLOCKED(ch, obj, door)	((obj) ? \
			(!OBJVAL_FLAGGED(obj, CONT_LOCKED)) :\
			(!EXIT_FLAGGED(EXIT(ch, door), EX_LOCKED)))
#define DOOR_IS_PICKPROOF(ch, obj, door) ((obj) ? \
			(OBJVAL_FLAGGED(obj, CONT_PICKPROOF)) : \
			(EXIT_FLAGGED(EXIT(ch, door), EX_PICKPROOF)))

#define DOOR_IS_CLOSED(ch, obj, door)	(!(DOOR_IS_OPEN(ch, obj, door)))
#define DOOR_IS_LOCKED(ch, obj, door)	(!(DOOR_IS_UNLOCKED(ch, obj, door)))
#define DOOR_KEY(ch, obj, door)		((obj) ? (GET_OBJ_VAL(obj, 2)) : \
					(EXIT(ch, door)->key))

ACMD(do_gen_door)
{
	int door = -1;
	obj_vnum keynum;
	char type[MAX_INPUT_LENGTH], dir[MAX_INPUT_LENGTH];
	struct obj_data *obj = NULL;
	struct char_data *victim = NULL;

	skip_spaces(&argument);
	if (!*argument) {
		send_to_char(ch, "%c%s what?\r\n", UPPER(*cmd_door[subcmd]), cmd_door[subcmd] + 1);
		return;
	}
	two_arguments(argument, type, dir);
	if (!generic_find(type, FIND_OBJ_INV | FIND_OBJ_ROOM, ch, &victim, &obj))
		door = find_door(ch, type, dir, cmd_door[subcmd]);

	if ((obj) || (door >= 0)) {
		keynum = DOOR_KEY(ch, obj, door);
		if (!(DOOR_IS_OPENABLE(ch, obj, door)))
			act("You can't $F that!", FALSE, ch, 0, cmd_door[subcmd], TO_CHAR);
		else if (!DOOR_IS_OPEN(ch, obj, door) &&
					(flags_door[subcmd].NEED_OPEN))
			send_to_char(ch, "But it's already closed!\r\n");
		else if (!DOOR_IS_CLOSED(ch, obj, door) &&
					(flags_door[subcmd].NEED_CLOSED))
			send_to_char(ch, "But it's currently open!\r\n");
		else if (!(DOOR_IS_LOCKED(ch, obj, door)) &&
					(flags_door[subcmd].NEED_LOCKED))
			send_to_char(ch, "Oh.. it wasn't locked, after all..\r\n");
		else if (!(DOOR_IS_UNLOCKED(ch, obj, door)) &&
					(flags_door[subcmd].NEED_UNLOCKED))
			send_to_char(ch, "It seems to be locked.\r\n");
		else if (!has_key(ch, keynum) && (GET_LEVEL(ch) < LVL_GOD) &&
					((subcmd == SCMD_LOCK) || (subcmd == SCMD_UNLOCK)))
			send_to_char(ch, "You don't seem to have the proper key.\r\n");
		else if (ok_pick(ch, keynum, DOOR_IS_PICKPROOF(ch, obj, door), subcmd))
			do_doorcmd(ch, obj, door, subcmd);
	}
	return;
}



ACMD(do_enter)
{
	char buf[MAX_INPUT_LENGTH];
	int door;

	one_argument(argument, buf);

	if (*buf) {			/* an argument was supplied, search for door
							 * keyword */
		for (door = 0; door < NUM_OF_DIRS; door++)
			if (EXIT(ch, door))
				if (EXIT(ch, door)->keyword)
					if (!str_cmp(EXIT(ch, door)->keyword, buf)) {
						perform_move(ch, door, 1);
						return;
					}
		send_to_char(ch, "There is no %s here.\r\n", buf);
	} else if (ROOM_FLAGGED(IN_ROOM(ch), ROOM_INDOORS))
		send_to_char(ch, "You are already indoors.\r\n");
	else {
		/* try to locate an entrance */
		for (door = 0; door < NUM_OF_DIRS; door++)
			if (EXIT(ch, door))
				if (EXIT(ch, door)->to_room != NOWHERE)
					if (!EXIT_FLAGGED(EXIT(ch, door), EX_CLOSED) &&
						 ROOM_FLAGGED(EXIT(ch, door)->to_room, ROOM_INDOORS)) {
						perform_move(ch, door, 1);
						return;
					}
		send_to_char(ch, "You can't seem to find anything to enter.\r\n");
	}
}


ACMD(do_leave)
{
	int door;

	if (OUTSIDE(ch))
		send_to_char(ch, "You are outside.. where do you want to go?\r\n");
	else {
		for (door = 0; door < NUM_OF_DIRS; door++)
			if (EXIT(ch, door))
				if (EXIT(ch, door)->to_room != NOWHERE)
					if (!EXIT_FLAGGED(EXIT(ch, door), EX_CLOSED) &&
						 !ROOM_FLAGGED(EXIT(ch, door)->to_room, ROOM_INDOORS)) {
						perform_move(ch, door, 1);
						return;
					}
		send_to_char(ch, "I see no obvious exits to the outside.\r\n");
	}
}


ACMD(do_stand)
{
	switch (GET_POS(ch)) {
		case POS_STANDING:
			send_to_char(ch, "You are already standing.\r\n");
			break;
		case POS_SITTING:
			send_to_char(ch, "You stand up.\r\n");
			act("$n clambers to $s feet.", TRUE, ch, 0, 0, TO_ROOM);

			/* Were they sitting in a chair? */
			char_from_chair(ch); 

			/* Will be sitting after a successful bash and may still be fighting. */
			GET_POS(ch) = FIGHTING(ch) ? POS_FIGHTING : POS_STANDING;
			break;
		case POS_RESTING:
			send_to_char(ch, "You stop resting, and stand up.\r\n");
			act("$n stops resting, and clambers on $s feet.", TRUE, ch, 0, 0, TO_ROOM);

			/* Were they sitting in the chair */
			char_from_chair(ch);

			GET_POS(ch) = POS_STANDING;
			break;
		case POS_SLEEPING:
			send_to_char(ch, "You have to wake up first!\r\n");
			break;
		case POS_FIGHTING:
			send_to_char(ch, "Do you not consider fighting as standing?\r\n");
			break;
		case POS_FLYING:
			send_to_char(ch, "You stop flying and plant your feet on the ground.");
			act("$n seems to be more down to earth.", TRUE, ch, 0, 0, TO_ROOM);
			GET_POS(ch) = POS_STANDING;
			break;

		default:
			send_to_char(ch, "You stop floating around, and put your feet on the ground.\r\n");
			act("$n stops floating around, and puts $s feet on the ground.",
				 TRUE, ch, 0, 0, TO_ROOM);
			GET_POS(ch) = POS_STANDING;
			break;
	}
}


ACMD(do_sit)
{
	struct obj_data *chair;
	struct char_data *tempch;
	char arg[MAX_INPUT_LENGTH];
	int found;

	one_argument(argument, arg);

	if (!*arg)
		found = 0;

	if (!(chair = get_obj_in_list_vis(ch, arg, NULL, world[IN_ROOM(ch)].contents)))
		found = 0;
	else
		found = 1;


	switch (GET_POS(ch)) {
		case POS_STANDING:
			if (found == 0) {
				send_to_char(ch, "You sit down.\r\n");
				act("$n sits down.", FALSE, ch, 0, 0, TO_ROOM);
				GET_POS(ch) = POS_SITTING;
			} else {
				if (GET_OBJ_TYPE(chair) != ITEM_CHAIR){
					send_to_char(ch, "You can't sit in that!\r\n");
					return;
				} else if (GET_OBJ_VAL(chair, 1) > GET_OBJ_VAL(chair, 0)){
					/* val 1 is current number in chair, 0 is max in chair */
					act("$p looks like it's all full.", 
						 TRUE, ch, chair, 0, TO_CHAR);
					log("SYSERR: chair %d holding too many people.",
						 GET_OBJ_VNUM(chair));
					return;
				} else if (GET_OBJ_VAL(chair, 1) == GET_OBJ_VAL(chair, 0)){
					act("There is no where left to sit upon $p.", 
						 TRUE, ch, chair, 0, TO_CHAR);
					return;
				} else {
					if (OBJ_SAT_IN_BY(chair) == NULL)
						OBJ_SAT_IN_BY(chair) = ch;

					for (tempch = OBJ_SAT_IN_BY(chair); 
						  tempch != ch ; 
						  tempch = NEXT_SITTING(tempch)) {
						if (NEXT_SITTING(tempch))
							continue;
						NEXT_SITTING(tempch) = ch;
					}
					act("You sit down upon $p.", TRUE, ch, chair, 0, TO_CHAR);
					act("$n sits down upon $p.", TRUE, ch, chair, 0, TO_ROOM);
					SITTING(ch) = chair;
					NEXT_SITTING(ch) = NULL;
					GET_OBJ_VAL(chair, 1) += 1;
					GET_POS(ch) = POS_SITTING;
				}
			}
			break;
		case POS_SITTING:
			send_to_char(ch, "You're sitting already.\r\n");
			break;
		case POS_RESTING:
			send_to_char(ch, "You stop resting, and sit up.\r\n");
			act("$n stops resting.", TRUE, ch, 0, 0, TO_ROOM);
			GET_POS(ch) = POS_SITTING;
			break;
		case POS_SLEEPING:
			send_to_char(ch, "You have to wake up first.\r\n");
			break;
		case POS_FIGHTING:
			send_to_char(ch, "Sit down while fighting? Are you MAD?\r\n");
			break;
		case POS_FLYING:
			send_to_char(ch, "You can't sit while flying.\r\n");
			break;
		default:
			send_to_char(ch, "You stop floating around, and sit down.\r\n");
			act("$n stops floating around, and sits down.", TRUE, ch, 0, 0, TO_ROOM);
			GET_POS(ch) = POS_SITTING;
			break;
	}
}


ACMD(do_rest)
{
	switch (GET_POS(ch)) {
		case POS_STANDING:
			send_to_char(ch, "You sit down and rest your tired bones.\r\n");
			act("$n sits down and rests.", TRUE, ch, 0, 0, TO_ROOM);
			GET_POS(ch) = POS_RESTING;
			break;
		case POS_SITTING:
			send_to_char(ch, "You rest your tired bones.\r\n");
			act("$n rests.", TRUE, ch, 0, 0, TO_ROOM);
			GET_POS(ch) = POS_RESTING;
			break;
		case POS_RESTING:
			send_to_char(ch, "You are already resting.\r\n");
			break;
		case POS_SLEEPING:
			send_to_char(ch, "You have to wake up first.\r\n");
			break;
		case POS_FIGHTING:
			send_to_char(ch, "Rest while fighting?  Are you MAD?\r\n");
			break;
		case POS_FLYING:
			send_to_char(ch, "You fold your legs and hover in meditation.\r\n");
			act("$n folds $s legs and hovers, meditating.",
				 TRUE, ch, 0, 0, TO_ROOM);
			GET_POS(ch) = POS_RESTING;
			break;
		default:
			send_to_char(ch, "You stop floating around, and stop to rest your tired bones.\r\n");
			act("$n stops floating around, and rests.", FALSE, ch, 0, 0, TO_ROOM);
			GET_POS(ch) = POS_SITTING;
			break;
	}
}


ACMD(do_sleep)
{
	switch (GET_POS(ch)) {
		case POS_STANDING:
		case POS_SITTING:
		case POS_RESTING:
			send_to_char(ch, "You go to sleep.\r\n");
			act("$n lies down and falls asleep.", TRUE, ch, 0, 0, TO_ROOM);
			GET_POS(ch) = POS_SLEEPING;
			break;
		case POS_SLEEPING:
			send_to_char(ch, "You are already sound asleep.\r\n");
			break;
		case POS_FIGHTING:
			send_to_char(ch, "Sleep while fighting?  Are you MAD?\r\n");
			break;
		case POS_FLYING:
			send_to_char(ch, "You float to the ground and fall asleep.\r\n");
			act("$n floats to the ground and falls asleep.", 
				 TRUE, ch, 0, 0, TO_ROOM);
			GET_POS(ch) = POS_SLEEPING;
			break;
		default:
			send_to_char(ch, "You stop floating around, and lie down to sleep.\r\n");
			act("$n stops floating around, and lie down to sleep.",
				 TRUE, ch, 0, 0, TO_ROOM);
			GET_POS(ch) = POS_SLEEPING;
			break;
	}
}


ACMD(do_wake)
{
	char arg[MAX_INPUT_LENGTH];
	struct char_data *vict;
	int self = 0;

	one_argument(argument, arg);
	if (*arg) {
		if (GET_POS(ch) == POS_SLEEPING)
			send_to_char(ch, "Maybe you should wake yourself up first.\r\n");
		else if ((vict = get_char_vis(ch, arg, NULL, FIND_CHAR_ROOM)) == NULL)
			send_to_char(ch, "%s", NOPERSON);
		else if (vict == ch)
			self = 1;
		else if (AWAKE(vict))
			act("$E is already awake.", FALSE, ch, 0, vict, TO_CHAR);
		else if (AFF_FLAGGED(vict, AFF_SLEEP))
			act("You can't wake $M up!", FALSE, ch, 0, vict, TO_CHAR);
		else if (GET_POS(vict) < POS_SLEEPING)
			act("$E's in pretty bad shape!", FALSE, ch, 0, vict, TO_CHAR);
		else {
			act("You wake $M up.", FALSE, ch, 0, vict, TO_CHAR);
			act("You are awakened by $n.", FALSE, ch, 0, vict, TO_VICT | TO_SLEEP);
			GET_POS(vict) = POS_SITTING;
		}
		if (!self)
			return;
	}
	if (AFF_FLAGGED(ch, AFF_SLEEP))
		send_to_char(ch, "You can't wake up!\r\n");
	else if (GET_POS(ch) > POS_SLEEPING)
		send_to_char(ch, "You are already awake...\r\n");
	else {
		send_to_char(ch, "You awaken, and sit up.\r\n");
		act("$n awakens.", TRUE, ch, 0, 0, TO_ROOM);
		GET_POS(ch) = POS_SITTING;
	}
}


ACMD(do_follow)
{
	char buf[MAX_INPUT_LENGTH];
	struct char_data *leader;

	one_argument(argument, buf);

	if (*buf) {
		if (!(leader = get_char_vis(ch, buf, NULL, FIND_CHAR_ROOM))) {
			send_to_char(ch, "%s", NOPERSON);
			return;
		}
	} else {
		send_to_char(ch, "Whom do you wish to follow?\r\n");
		return;
	}

	if (ch->master == leader) {
		act("You are already following $M.", FALSE, ch, 0, leader, TO_CHAR);
		return;
	}
	if (AFF_FLAGGED(ch, AFF_CHARM) && (ch->master)) {
		act("But you only feel like following $N!", FALSE, ch, 0, ch->master, TO_CHAR);
	} else {			/* Not Charmed follow person */
		if (leader == ch) {
			if (!ch->master) {
				send_to_char(ch, "You are already following yourself.\r\n");
				return;
			}
			stop_follower(ch);
		} else {
			if (circle_follow(ch, leader)) {
				send_to_char(ch, "Sorry, but following in loops is not allowed.\r\n");
				return;
			}
			if (ch->master)
				stop_follower(ch);
			AFF_FLAGS(ch).AFF_GROUP = 0;
			add_follower(ch, leader);
		}
	}
}


/* Mounts (DAK) */
ACMD(do_mount) {
   char arg[MAX_INPUT_LENGTH];
   struct char_data *vict;
	int goal = GET_SKILL(ch, SKILL_MOUNT);
   
   one_argument(argument, arg);
   
   if (!arg || !*arg) {
		send_to_char(ch, "Mount who?\r\n");
		return;
   } else if (!(vict = get_char_room_vis(ch, arg, NULL))) {
		send_to_char(ch, "There is no-one by that name here.\r\n");
		return;
   } else if (!IS_NPC(vict) && GET_LEVEL(ch) < LVL_IMMORT) {
		send_to_char(ch, "Ehh... no.\r\n");
		return;
   } else if (RIDING(ch) || RIDDEN_BY(ch)) {
		send_to_char(ch, "You are already mounted.\r\n");
		return;
   } else if (RIDING(vict) || RIDDEN_BY(vict)) {
		send_to_char(ch, "It is already mounted.\r\n");
		return;
   } else if (GET_LEVEL(ch) < LVL_IMMORT && IS_NPC(vict) && 
				  !MOB_FLAGGED(vict, MOB_MOUNTABLE)) {
		send_to_char(ch, "You can't mount that!\r\n");
		return;
   } else if (!goal) {
		send_to_char(ch, "First you need to learn *how* to mount.\r\n");
		return;
   } else if (goal <= rand_number(1, 101)) {
		act("You try to mount $N, but slip and fall off.", 
			 FALSE, ch, 0, vict, TO_CHAR);
		act("$n tries to mount you, but slips and falls off.",
			 FALSE, ch, 0, vict, TO_VICT);
		act("$n tries to mount $N, but slips and falls off.", 
			 TRUE, ch, 0, vict, TO_NOTVICT);
		damage(ch, ch, dice(1, 2), -1);
		return;
   }
   
   act("You mount $N.", FALSE, ch, 0, vict, TO_CHAR);
   act("$n mounts you.", FALSE, ch, 0, vict, TO_VICT);
   act("$n mounts $N.", TRUE, ch, 0, vict, TO_NOTVICT);
   mount_char(ch, vict);
   
   if (IS_NPC(vict) && 
		 !AFF_FLAGGED(vict, AFF_TAMED) &&
		 goal <= rand_number(1, 101)) {

		act("$N suddenly bucks upwards, throwing you violently to the ground!", 
			 FALSE, ch, 0, vict, TO_CHAR);
		act("$n is thrown to the ground as $N violently bucks!",
			 TRUE, ch, 0, vict, TO_NOTVICT);
		act("You buck violently and throw $n to the ground.", 
			 FALSE, ch, 0, vict, TO_VICT);

		dismount_char(ch);
		damage(vict, ch, dice(1,3), -1);
   }
}
 
 
ACMD(do_dismount) {
   if (!RIDING(ch)) {
		send_to_char(ch, "You aren't even riding anything.\r\n");
		return;
   } else if (SECT(IN_ROOM(ch)) == SECT_WATER_NOSWIM && !has_boat(ch)) {
		send_to_char(ch, "Yah, right, and then drown...\r\n");
		return;
   }
   
   act("You dismount $N.", FALSE, ch, 0, RIDING(ch), TO_CHAR);
   act("$n dismounts from you.", FALSE, ch, 0, RIDING(ch), TO_VICT);
   act("$n dismounts $N.", TRUE, ch, 0, RIDING(ch), TO_NOTVICT);
   dismount_char(ch);
}
 
 
ACMD(do_buck) {
   if (!RIDDEN_BY(ch)) {
		send_to_char(ch, "You're not even being ridden!\r\n");
		return;
   } else if (AFF_FLAGGED(ch, AFF_TAMED)) {
		send_to_char(ch, "But you're tamed!\r\n");
		return;
   }
   
   act("You quickly buck, throwing $N to the ground.", 
		 FALSE, ch, 0, RIDDEN_BY(ch), TO_CHAR);
   act("$n quickly bucks, throwing you to the ground.", 
		 FALSE, ch, 0, RIDDEN_BY(ch), TO_VICT);
   act("$n quickly bucks, throwing $N to the ground.", 
		 FALSE, ch, 0, RIDDEN_BY(ch), TO_NOTVICT);
   GET_POS(RIDDEN_BY(ch)) = POS_SITTING;

   if (rand_number(0, 4)) {
		send_to_char(RIDDEN_BY(ch), "You hit the ground hard!\r\n");
		damage(RIDDEN_BY(ch), RIDDEN_BY(ch), dice(2,4), -1);
   }
   dismount_char(ch);
   
   /* you might want to call set_fighting() or some non-sense
	 * here if you want the mount to attack the unseated rider
	 * or vice-versa. */
}
 
 
ACMD(do_tame) {
   char arg[MAX_INPUT_LENGTH];
   struct affected_type af;
   struct char_data *vict;
	int goal = GET_SKILL(ch, SKILL_TAME);
   
   one_argument(argument, arg);
   
   if (!arg || !*arg) {
		send_to_char(ch, "Tame who?\r\n");
		return;
   } else if (!(vict = get_char_room_vis(ch, arg, NULL))) {
		send_to_char(ch, "They're not here.\r\n");
		return;
   } else if (GET_LEVEL(ch) < LVL_IMMORT && IS_NPC(vict) && 
				  !MOB_FLAGGED(vict, MOB_MOUNTABLE)) {
		send_to_char(ch, "You can't do that to them.\r\n");
		return;
   } else if (!goal) {
		send_to_char(ch, "You don't even know how to tame something.\r\n");
		return;
   } else if (!IS_NPC(vict) && GET_LEVEL(ch) < LVL_IMMORT) {
		send_to_char(ch, "You can't do that.\r\n");
		return;
   } else if (goal <= rand_number(1, 101)) {
		send_to_char(ch, "You fail to tame it.\r\n");
		return;
   }
   
   af.type = SKILL_TAME;
   af.duration = 24;
   af.modifier = 0;
   af.location = APPLY_NONE;
   af.bitvector.AFF_TAMED = 1;
   affect_join(vict, &af, FALSE, FALSE, FALSE, FALSE);
   
   act("You tame $N.", FALSE, ch, 0, vict, TO_CHAR);
   act("$n tames you.", FALSE, ch, 0, vict, TO_VICT);
   act("$n tames $N.", FALSE, ch, 0, vict, TO_NOTVICT);
}


room_rnum get_house_vnum(struct char_data *ch)
{
	int i;

	for (i = 0; i < num_of_houses; i++) {
		if (GET_IDNUM(ch) == house_control[i].owner)
			return (house_control[i].vnum);
	}
	
	return (NOWHERE);
}


room_rnum get_home(struct char_data *ch)
{
	room_rnum load_room;

	if (PLR_FLAGGED(ch, PLR_FROZEN))
		return (r_frozen_start_room);

	if ((load_room = get_house_vnum(ch)) != NOWHERE)
		return (load_room);
	
	if (GET_LEVEL(ch) >= LVL_IMMORT)
		return (r_immort_start_room);
	else
		return (r_mortal_start_room);
}


ACMD(do_recall)
{
	int cp, cost;
	room_rnum where = r_mortal_start_room;

	if (!can_recall(ch))
		return;

	if (allow_recall == 0) {
		send_to_char(ch, "NO!  Use the spell or a scroll.\r\n");
		return;
	}
	
	where = get_home(ch);

	if (allow_recall > 1 && GET_LEVEL(ch) >= allow_recall) {
		cp = convert_all_to_copper(ch);
		cost = GET_LEVEL(ch) * 1000; /* Change to suit your own tastes. */
		
		send_to_char(ch, "Recalling for %s%d%s copper...\r\n", QMAG, cost, QNRM);
		if (cp < cost) {
			send_to_char(ch, "You don't have enough money!\r\n");
			act("$n tried to recall but failed!", TRUE, ch, 0, 0, TO_ROOM);
			return;
      } else
			make_change(ch, cost, BUYING);
	} else if (allow_recall == 1 || GET_LEVEL(ch) < allow_recall)
		send_to_char(ch, "Recalling for FREE..\r\n");
	else
		mudlog(NRM, LVL_GOD, TRUE, "SYSERR: Problem with do_recall! "
				 "(allow_recall = %d)", allow_recall);

	act("$n disappears in a puff of smoke!", TRUE, ch, 0, 0, TO_ROOM);
	char_from_room(ch);
	char_to_room(ch, where);
	act("$n appears in the middle of the room.", TRUE, ch, 0, 0, TO_ROOM);
	look_at_room(ch, 0);
}
