/* ************************************************************************
 *   File: act.offensive.c                               Part of CircleMUD *
 *  Usage: player-level commands of an offensive nature                    *
 *                                                                         *
 *  All rights reserved.  See license.doc for complete information.        *
 *                                                                         *
 *  Copyright (C) 1993, 94 by the Trustees of the Johns Hopkins University *
 *  CircleMUD is based on DikuMUD, Copyright (C) 1990, 1991.               *
 ************************************************************************ */

#include "conf.h"
#include "sysdep.h"

#include "structs.h"
#include "utils.h"

#include "comm.h"
#include "constants.h"
#include "db.h"
#include "handler.h"
#include "interpreter.h"
#include "spells.h"

/* extern variables */
extern int pk_allowed;
extern int max_npc_corpse_time, max_pc_corpse_time;

/* extern functions */
void raw_kill(struct char_data *ch, struct char_data *killer);
void check_killer(struct char_data *ch, struct char_data *vict);
int compute_armor_class(struct char_data *ch);
void fire_missile(struct char_data *ch, char arg1[MAX_INPUT_LENGTH],
         struct obj_data *missile, int pos, int range, int dir);

/* local functions */
ACMD(do_assist);
ACMD(do_bash);
ACMD(do_disarm);
ACMD(do_draw);
ACMD(do_flee);
ACMD(do_gut);
ACMD(do_hit);
ACMD(do_kick);
ACMD(do_kill);
ACMD(do_order);
ACMD(do_rescue);
ACMD(do_retreat);
ACMD(do_shoot);
ACMD(do_stun);
ACMD(do_throw);
ACMD(do_whirlwind);


ACMD(do_assist)
{
	char arg[MAX_INPUT_LENGTH];
	struct char_data *helpee, *opponent;

	if (FIGHTING(ch)) {
		send_to_char(ch, "You're already fighting!  How can you assist someone else?\r\n");
		return;
	}
	one_argument(argument, arg);

	if (!*arg)
		send_to_char(ch, "Whom do you wish to assist?\r\n");
	else if (!(helpee = get_char_vis(ch, arg, NULL, FIND_CHAR_ROOM)))
		send_to_char(ch, "%s", NOPERSON);
	else if (helpee == ch)
		send_to_char(ch, "You can't help yourself any more than this!\r\n");
	else {
		/*
		 * Hit the same enemy the person you're helping is.
		 */
		if (FIGHTING(helpee))
			opponent = FIGHTING(helpee);
		else
			for (opponent = world[IN_ROOM(ch)].people;
				  opponent && (FIGHTING(opponent) != helpee);
				  opponent = opponent->next_in_room)
				;

		if (!opponent)
			act("But nobody is fighting $M!", FALSE, ch, 0, helpee, TO_CHAR);
		else if (!CAN_SEE(ch, opponent))
			act("You can't see who is fighting $M!", FALSE, ch, 0, helpee, TO_CHAR);
		else if (!pk_allowed && !IS_NPC(opponent))	/* prevent accidental pkill */
			act("Use 'murder' if you really want to attack $N.", FALSE,
				 ch, 0, opponent, TO_CHAR);
		else {
			send_to_char(ch, "You join the fight!\r\n");
			act("$N assists you!", 0, helpee, 0, ch, TO_CHAR);
			act("$n assists $N.", FALSE, ch, 0, helpee, TO_NOTVICT);
			hit(ch, opponent, TYPE_UNDEFINED);
		}
	}
}


ACMD(do_hit)
{
	char arg[MAX_INPUT_LENGTH];
	struct char_data *vict;
	struct follow_type *k;

	one_argument(argument, arg);

	if (!*arg)
		send_to_char(ch, "Hit who?\r\n");
	else if (!(vict = get_char_vis(ch, arg, NULL, FIND_CHAR_ROOM)))
		send_to_char(ch, "They don't seem to be here.\r\n");
	else if (vict == ch) {
		send_to_char(ch, "You hit yourself...OUCH!.\r\n");
		act("$n hits $mself, and says OUCH!", FALSE, ch, 0, vict, TO_ROOM);
	} else if (AFF_FLAGGED(ch, AFF_CHARM) && (ch->master == vict))
		act("$N is just such a good friend, you simply can't hit $M.", FALSE, ch, 0, vict, TO_CHAR);
	else {
		if (!pk_allowed) {
			if (!IS_NPC(vict) && !IS_NPC(ch)) {
				if (subcmd != SCMD_MURDER) {
					send_to_char(ch, "Use 'murder' to hit another player.\r\n");
					return;
				} else {
					check_killer(ch, vict);
				}
			}
			if (AFF_FLAGGED(ch, AFF_CHARM) && !IS_NPC(ch->master) && !IS_NPC(vict))
				return;			/* you can't order a charmed pet to attack a
									 * player */
		}
		if ((GET_POS(ch) == POS_STANDING) && (vict != FIGHTING(ch))) {
			hit(ch, vict, TYPE_UNDEFINED);

			/* Auto assist! */
			for (k = ch->followers; k; k = k->next) {
				if (PRF_FLAGGED(k->follower, PRF_AUTOASSIST) &&
					 (IN_ROOM(k->follower) == IN_ROOM(ch)))
					do_assist(k->follower, GET_NAME(ch), 0, 0);
			}

			WAIT_STATE(ch, PULSE_VIOLENCE + 2);
		} else
			send_to_char(ch, "You do the best you can!\r\n");
	}
}



ACMD(do_kill)
{
	char arg[MAX_INPUT_LENGTH];
	struct char_data *vict;

	if (GET_LEVEL(ch) < LVL_IMPL || IS_NPC(ch)) {
		do_hit(ch, argument, cmd, subcmd);
		return;
	}
	one_argument(argument, arg);

	if (!*arg) {
		send_to_char(ch, "Kill who?\r\n");
	} else {
		if (!(vict = get_char_vis(ch, arg, NULL, FIND_CHAR_ROOM)))
			send_to_char(ch, "They aren't here.\r\n");
		else if (ch == vict)
			send_to_char(ch, "Your mother would be so sad.. :(\r\n");
		else {
			act("You chop $M to pieces!  Ah!  The blood!", FALSE, ch, 0, vict, TO_CHAR);
			act("$N chops you to pieces!", FALSE, vict, 0, ch, TO_CHAR);
			act("$n brutally slays $N!", FALSE, ch, 0, vict, TO_NOTVICT);
			raw_kill(vict, ch);
		}
	}
}


ACMD(do_order)
{
	char name[MAX_INPUT_LENGTH], message[MAX_INPUT_LENGTH];
	bool found = FALSE;
	struct char_data *vict;
	struct follow_type *k;

	half_chop(argument, name, message);

	if (!*name || !*message)
		send_to_char(ch, "Order who to do what?\r\n");
	else if (!(vict = get_char_vis(ch, name, NULL, FIND_CHAR_ROOM)) && !is_abbrev(name, "followers"))
		send_to_char(ch, "That person isn't here.\r\n");
	else if (ch == vict)
		send_to_char(ch, "You obviously suffer from skitzofrenia.\r\n");
	else {
		if (AFF_FLAGGED(ch, AFF_CHARM)) {
			send_to_char(ch, "Your superior would not aprove of you giving orders.\r\n");
			return;
		}
		if (vict) {
			char buf[MAX_STRING_LENGTH];

			snprintf(buf, sizeof(buf), "$N orders you to '%s'", message);
			act(buf, FALSE, vict, 0, ch, TO_CHAR);
			act("$n gives $N an order.", FALSE, ch, 0, vict, TO_ROOM);

			if ((vict->master != ch) || !AFF_FLAGGED(vict, AFF_CHARM))
				act("$n has an indifferent look.", FALSE, vict, 0, 0, TO_ROOM);
			else {
				send_to_char(ch, "%s", OK);
				command_interpreter(vict, message);
			}
		} else {			/* This is order "followers" */
			char buf[MAX_STRING_LENGTH];

			snprintf(buf, sizeof(buf), "$n issues the order '%s'.", message);
			act(buf, FALSE, ch, 0, 0, TO_ROOM);

			for (k = ch->followers; k; k = k->next) {
				if (IN_ROOM(ch) == IN_ROOM(k->follower))
					if (AFF_FLAGGED(k->follower, AFF_CHARM)) {
						found = TRUE;
						command_interpreter(k->follower, message);
					}
			}
			if (found)
				send_to_char(ch, "%s", OK);
			else
				send_to_char(ch, "Nobody here is a loyal subject of yours!\r\n");
		}
	}
}



ACMD(do_flee)
{
	int i, attempt, loss;
	struct char_data *was_fighting;

	if (GET_POS(ch) < POS_FIGHTING) {
		send_to_char(ch, "You are in pretty bad shape, unable to flee!\r\n");
		return;
	}

	for (i = 0; i < 6; i++) {
		attempt = rand_number(0, NUM_OF_DIRS - 1);	/* Select a random direction */
		if (CAN_GO(ch, attempt) &&
			 !ROOM_FLAGGED(EXIT(ch, attempt)->to_room, ROOM_DEATH)) {
			act("$n panics, and attempts to flee!", TRUE, ch, 0, 0, TO_ROOM);
			was_fighting = FIGHTING(ch);
			if (do_simple_move(ch, attempt, TRUE)) {
				send_to_char(ch, "You flee head over heels.\r\n");
				if (was_fighting && !IS_NPC(ch)) {
					loss = GET_MAX_HIT(was_fighting) - GET_HIT(was_fighting);
					loss *= GET_LEVEL(was_fighting);
					gain_exp(ch, -loss);
				}
			} else {
				act("$n tries to flee, but can't!", TRUE, ch, 0, 0, TO_ROOM);
			}
			return;
		}
	}
	send_to_char(ch, "PANIC!  You couldn't escape!\r\n");
}


ACMD(do_bash)
{
	char arg[MAX_INPUT_LENGTH];
	struct char_data *vict;
	int roll = rand_number(1, 101);	/* 101% is a complete failure */
	int goal = GET_SKILL(ch, SKILL_BASH);

	one_argument(argument, arg);

	if (IS_NPC(ch) || !goal) {
		send_to_char(ch, "You have no idea how.\r\n");
		return;
	}
	if (ROOM_FLAGGED(IN_ROOM(ch), ROOM_PEACEFUL)) {
		send_to_char(ch, "This room just has such a peaceful, easy feeling...\r\n");
		return;
	}
	if (!GET_EQ(ch, WEAR_WIELD)) {
		send_to_char(ch, "You need to wield a weapon to make it a success.\r\n");
		return;
	}
	if (!(vict = get_char_vis(ch, arg, NULL, FIND_CHAR_ROOM))) {
		if (FIGHTING(ch) && IN_ROOM(ch) == IN_ROOM(FIGHTING(ch))) {
			vict = FIGHTING(ch);
		} else {
			send_to_char(ch, "Bash who?\r\n");
			return;
		}
	}
	if (vict == ch) {
		send_to_char(ch, "Aren't we funny today...\r\n");
		return;
	}

	if (MOB_FLAGGED(vict, MOB_NOBASH))
		roll = 101;

	if (roll > goal) {
		damage(ch, vict, 0, SKILL_BASH);
		GET_POS(ch) = POS_SITTING;
	} else {
		/*
		 * If we bash a player and they wimp out, they will move to the
		 * previous room before we set them sitting.  If we try to set
		 * the victim sitting first to make sure they don't flee, then
		 * we can't bash them!  So now we only set them sitting if they
		 * didn't flee. -gg 9/21/98
		 */
		if (damage(ch, vict, 1, SKILL_BASH) > 0) {	/* -1 = dead, 0 = miss */
			WAIT_STATE(vict, PULSE_VIOLENCE);
			if (IN_ROOM(ch) == IN_ROOM(vict))
				GET_POS(vict) = POS_SITTING;
		}
	}
	WAIT_STATE(ch, PULSE_VIOLENCE * 2);
}


ACMD(do_rescue)
{
	char arg[MAX_INPUT_LENGTH];
	struct char_data *vict, *tmp_ch;
	int roll = rand_number(1, 101);	/* 101% is a complete failure */
	int goal = GET_SKILL(ch, SKILL_RESCUE);

	if (IS_NPC(ch) || !goal) {
		send_to_char(ch, "You have no idea how to do that.\r\n");
		return;
	}

	one_argument(argument, arg);

	if (!(vict = get_char_vis(ch, arg, NULL, FIND_CHAR_ROOM))) {
		send_to_char(ch, "Whom do you want to rescue?\r\n");
		return;
	}
	if (vict == ch) {
		send_to_char(ch, "What about fleeing instead?\r\n");
		return;
	}
	if (FIGHTING(ch) == vict) {
		send_to_char(ch, "How can you rescue someone you are trying "
						 "to kill?\r\n");
		return;
	}
	for (tmp_ch = world[IN_ROOM(ch)].people; tmp_ch &&
			  (FIGHTING(tmp_ch) != vict); tmp_ch = tmp_ch->next_in_room);

	if (!tmp_ch) {
		act("But nobody is fighting $M!", FALSE, ch, 0, vict, TO_CHAR);
		return;
	}

	if (roll > goal) {
		send_to_char(ch, "You fail the rescue!\r\n");
		return;
	}
	send_to_char(ch, "Banzai!  To the rescue...\r\n");
	act("You are rescued by $N, you are confused!", 
		 FALSE, vict, 0, ch, TO_CHAR);
	act("$n heroically rescues $N!",
		 FALSE, ch, 0, vict, TO_NOTVICT);

	if (FIGHTING(vict) == tmp_ch)
		stop_fighting(vict);
	if (FIGHTING(tmp_ch))
		stop_fighting(tmp_ch);
	if (FIGHTING(ch))
		stop_fighting(ch);

	set_fighting(ch, tmp_ch);
	set_fighting(tmp_ch, ch);

	WAIT_STATE(vict, 2 * PULSE_VIOLENCE);
}


ACMD(do_kick)
{
	char arg[MAX_INPUT_LENGTH];
	struct char_data *vict;
	int roll, goal = GET_SKILL(ch, SKILL_KICK);

	if (IS_NPC(ch) || !goal) {
		send_to_char(ch, "You have no idea how.\r\n");
		return;
	}
	one_argument(argument, arg);

	if (!(vict = get_char_vis(ch, arg, NULL, FIND_CHAR_ROOM))) {
		if (FIGHTING(ch) && IN_ROOM(ch) == IN_ROOM(FIGHTING(ch)))
			vict = FIGHTING(ch);
		else {
			send_to_char(ch, "Kick who?\r\n");
			return;
		}
	}
	if (vict == ch) {
		send_to_char(ch, "Aren't we funny today...\r\n");
		return;
	}

	roll = ((10 - (compute_armor_class(vict) / 10)) * 2) + rand_number(1, 101);
	if (roll > goal)
		damage(ch, vict, 0, SKILL_KICK);
	else
		damage(ch, vict, GET_LEVEL(ch) / 2, SKILL_KICK);

	WAIT_STATE(ch, PULSE_VIOLENCE * 3);
}


/*
  From: Ryan Stapleton <raal@penn.com>
  Subject: stun skill

  Here is my stun code.  I use it for a Ranger on my mud.
  you must include AFF_STUN in structs.h in the AFF section.
*/
void stun_failure(struct char_data *ch)
{
	struct affected_type af;

	send_to_char(ch, "You fail.\r\n");
	if (rand_number(1, 100) <= 1) {
		if (GET_LEVEL(ch) < LVL_IMMORT) {
			send_to_char(ch, "Oops, You stuned your self.\r\n");
			af.bitvector.AFF_STUN = 1;
			af.duration = 0;
			affect_to_char(ch, &af);
			GET_POS(ch) = POS_STUNNED;
		}
	}
}


ACMD(do_stun)
{
	struct char_data *vict;
	struct affected_type af;
	char arg[MAX_INPUT_LENGTH];
	int diff_lev, chance;
	int roll = rand_number(1, 101);   /* 101% is a complete failure */
	int goal = GET_SKILL(ch, SKILL_STUN);

	one_argument(argument, arg);

	if (IS_NPC(ch) || !goal)
		send_to_char(ch, "You have no idea how!\r\n");
	else  if (!*arg)
		send_to_char(ch, "Stun who?\r\n");
	else if (!(vict = get_char_room_vis(ch, arg, NULL)))
		send_to_char(ch, "They don't seem to be here.\r\n");
	else if (!IS_NPC(vict) && !pk_allowed)
		send_to_char(ch, "You can not attack another PC.\r\n");
	else if (vict == ch) {
		send_to_char(ch, "You Stun yourself...OUCH!.\r\n");
		act("$n tries to stun $mself, and says OUCH!",
			 FALSE, ch, 0, vict, TO_ROOM);
	} else if (AFF_FLAGGED(ch, AFF_CHARM) && (ch->master == vict)) {
		act("$N is just such a good friend, you simply can't stun $M.", 
			 FALSE, ch, 0, vict, TO_CHAR);
	} else if (ROOM_FLAGGED(IN_ROOM(ch), ROOM_PEACEFUL)) 
		send_to_char(ch, "This room just has such a peaceful, easy feeling...\r\n");
	else if (GET_MOVE(ch) < 10) 
		send_to_char(ch, "You don't have the energy to do that.\r\n");
	else {
		roll = rand_number(1, 101);   /* 101% is a complete failure */
		goal = GET_SKILL(ch, SKILL_STUN);

		if (roll <= goal) {
			if (GET_LEVEL(ch) < LVL_IMMORT)
				GET_MOVE(ch) -= 10;

			diff_lev = GET_LEVEL(vict) - GET_LEVEL(ch);
			chance = 33 - (diff_lev);

			if (chance > 100)
				chance = 100;

			chance -= GET_DEX(vict);
			roll = rand_number(1, 101);

			if (chance > roll) {
				af.bitvector.AFF_STUN = 1;
				af.duration = 0;
				affect_to_char(vict, &af);

				stop_fighting(vict);
				GET_POS(vict) = POS_STUNNED;

				act("You stun $N.", FALSE, ch, 0, vict, TO_CHAR);
				act("$n stuns $N.", TRUE, ch, 0, vict, TO_NOTVICT);
				act("$n stuns you.", TRUE, ch, 0, ch->master, TO_VICT);
			} else /* if (chance > roll) { */
				stun_failure(ch);
		} else /* if (roll <= goal) { */
			stun_failure(ch);

		hit(ch, vict, TYPE_UNDEFINED);
		if (GET_LEVEL(ch) < LVL_IMMORT)
			WAIT_STATE(ch, PULSE_VIOLENCE * 4);
	}
}


ACMD(do_whirlwind)
{
	struct char_data *tch, *next_tch;
	char arg[MAX_INPUT_LENGTH];
	int goal = GET_SKILL(ch, SKILL_WHIRLWIND);
	int roll = rand_number(1, 101);

	one_argument(argument, arg);

	/* If player is a mob or skill isn't learned, we can't do this. */
	if (IS_NPC(ch) || !goal) {
		send_to_char(ch, "You have no idea how.\r\n");
		return;
	}

	/* Neither can we do this in a peaceful room */
	if (ROOM_FLAGGED(IN_ROOM(ch), ROOM_PEACEFUL)) {
		send_to_char(ch, "No, not in here...\r\n");
		return;
	}

	/* And finally, the show costs 30 moves, so the player must be able
	 * to pay */
	if (GET_LEVEL(ch) < LVL_IMMORT && GET_MOVE(ch) < 30) {
		send_to_char(ch, "You don't have the energy to do that just now!\r\n");
		return;
	}

	if (roll > goal) {
		send_to_char(ch, "You psyche yourself out to no avail!\r\n");
		return; 
	} else {
		/* Find first target, hit it, then move on to next one */
		for (tch = world[IN_ROOM(ch)].people; tch; tch = next_tch) {
			next_tch = tch->next_in_room;

			/*  We'll leave out immortals, players (for !pk muds) and
			 *  pets from the hit list
			 */
			if (tch == ch)
				continue;

			if (!IS_NPC(tch) && GET_LEVEL(tch) >= LVL_IMMORT)
				continue;

			if (!pk_allowed && !IS_NPC(ch) && !IS_NPC(tch))
				continue;

			if (!IS_NPC(ch) && IS_NPC(tch) && AFF_FLAGGED(tch, AFF_CHARM))
				continue; 

			/* GET_LEVEL(ch)*2 is the damage value, change if you like */
			damage(ch, tch, GET_LEVEL(ch) * 2, SKILL_WHIRLWIND);

			if (GET_LEVEL(ch) < LVL_IMMORT)
				GET_MOVE(ch) -= 30;
		}
	}
	
   WAIT_STATE(ch, PULSE_VIOLENCE * 2);
}



ACMD(do_retreat)
{
	int retreat_type, dir = 0;
	char arg[MAX_INPUT_LENGTH];
	int goal = GET_SKILL(ch, SKILL_RETREAT) / 2;
	int roll = rand_number(1, 101);

	if (IS_NPC(ch) || !goal) {
		send_to_char(ch, "Try FLEE instead!\r\n");
		return;
	}

	one_argument(argument, arg);

	if (!FIGHTING(ch)) {
		send_to_char(ch, "You are not fighting!\r\n");
		return;
   }
	
	if (!*arg) {
		send_to_char(ch, "Retreat where?!?\r\n");
		return;
   }

	retreat_type = search_block(argument + 1, dirs, FALSE);

	if (retreat_type < 0 || 
		 !EXIT(ch, retreat_type) ||
		 EXIT(ch, retreat_type)->to_room == NOWHERE) {
		send_to_char(ch, "Retreat where?\r\n");
		return;
   }

	if (roll <= goal) {
		if (CAN_GO(ch, dir) && !ROOM_FLAGGED(EXIT(ch,dir)->to_room, ROOM_DEATH)) {
			act("$n skillfully retreats from combat.", TRUE, ch, 0, 0, TO_ROOM);
			send_to_char(ch, "You skillfully retreat from combat.\r\n");

			if (FIGHTING(FIGHTING(ch)) == ch)
				stop_fighting(FIGHTING(ch));
			stop_fighting(ch);

			do_simple_move(ch, dir, TRUE);
			WAIT_STATE(ch, 2 * PULSE_VIOLENCE);
		} else {
			act("$n tries to retreat from combat but has no where to go!", 
				 TRUE, ch, 0, 0, TO_ROOM);
			send_to_char(ch, "You cannot retreat in that direction!");
			return;
		}
	} else {
		send_to_char(ch, "You fail your attempt to retreat!\r\n");
		WAIT_STATE(ch, PULSE_VIOLENCE);
		return;
	}
}


ACMD(do_disarm)
{
	struct char_data *vict;
	struct obj_data *weap; 
	char arg[MAX_INPUT_LENGTH];
	int roll = rand_number(1, 101);
	int goal = GET_SKILL(ch, SKILL_DISARM);

	if (IS_NPC(ch) || !goal) {
		send_to_char(ch, "You have no idea how.\r\n");
		return;
	}
	if (ROOM_FLAGGED(IN_ROOM(ch), ROOM_PEACEFUL)) {
		send_to_char(ch, "This room just has such a peaceful, easy "
						 "feeling...\r\n");
		return;
	}

	one_argument(argument, arg);

	if (!(vict = get_char_vis(ch, arg, NULL, FIND_CHAR_ROOM))) {
		if (FIGHTING(ch) && IN_ROOM(ch) == IN_ROOM(FIGHTING(ch)))
			vict = FIGHTING(ch);

		if (!vict) {
			send_to_char(ch, "Disarm who?!\r\n");
			return;
		}
	}

	if (!GET_EQ(ch, WEAR_WIELD)) {
		send_to_char(ch, "You need to wield a weapon yourself to disarm!\r\n");
		return;
	}

	if (vict == ch) {
		send_to_char(ch, "Just use 'remove <weapon>' instead!\r\n");
		return;
	}

	if (!(weap = GET_EQ(vict, WEAR_WIELD))) {
		send_to_char(ch, "They're not wielding a weapon!\r\n");
		return;      
	}

	/* Some modifiers, just to make it a bit more interesting. */
	goal += GET_DEX(ch);
	goal -= GET_DEX(vict);
	goal -= GET_LEVEL(vict);
	goal -= GET_OBJ_WEIGHT(weap);
	if (goal < 1)
		goal = 1;
	
	if (GET_LEVEL(ch) >= LVL_IMMORT)  
		roll = 0;

	if (roll <= goal) { 	/* Disarm */
		act("You knock $p from $N's hand!", 
			 FALSE, ch, weap, vict, TO_CHAR);
		act("$n knocks $p from your hand!", 
			 FALSE, ch, weap, vict, TO_VICT); 
		act("$n knocks $p from $N's hand!", 
			 FALSE, ch, weap, vict, TO_ROOM);
		obj_to_char(unequip_char(vict, WEAR_WIELD), vict);
	} else {
		act("You fail to disarm $N", FALSE, ch, weap, vict, TO_CHAR);
		act("$n fails to disarm you", FALSE, ch, weap, vict, TO_VICT);
		act("$n fails to disarm $N", FALSE, ch, weap, vict, TO_ROOM);
	}
	WAIT_STATE(ch, PULSE_VIOLENCE);
}



ACMD(do_gut)
{
	struct char_data *vict;
	struct obj_data *piece;
	int goal = GET_SKILL(ch, SKILL_GUT);
	int roll = rand_number(1, 101);
	char arg[MAX_INPUT_LENGTH];

	if (IS_NPC(ch) || !goal) {
		send_to_char(ch, "You are doing the best you can!\r\n");
		return;
	}

	one_argument(argument, arg);

	if (!(vict = get_char_vis(ch, arg, NULL, FIND_CHAR_ROOM))) {
		if (FIGHTING(ch) && IN_ROOM(ch) == IN_ROOM(FIGHTING(ch)))
			vict = FIGHTING(ch);

		if (!vict) {
			send_to_char(ch, "Gut who?!\r\n");
			return;
		}
	}
	
	if (vict == ch) {
		send_to_char(ch, "Aren't we funny today...\r\n");
		return;
	}

	if (!GET_EQ(ch, WEAR_WIELD)) {
		send_to_char(ch, "You need to wield a weapon to make it a success.\r\n");
		return;
	}

	if (GET_OBJ_VAL(GET_EQ(ch, WEAR_WIELD), 3) != TYPE_SLASH - TYPE_HIT &&
		 GET_OBJ_VAL(GET_EQ(ch, WEAR_WIELD), 3) != TYPE_CLAW - TYPE_HIT) {
		send_to_char(ch, "Only slashing weapons can be used for gutting.\r\n");
		return;
	}

	if (GET_POS(vict) > POS_INCAP) {
		send_to_char(ch, "They are not hurt enough for you to attempt that.\r\n");
		return;
	}

	if (roll > goal) {
		send_to_char(ch, "Even in %s's bad state, they manage to avoid your "
						 "wild slash.\r\n", GET_NAME(vict));
		send_to_char(vict, "You avoid a wild slash at your midsection.\r\n");
	} else {

		/* EWWWW */
		GET_HIT(vict) = -50; /* Make sure they die... */
		act("You gut $N!", FALSE, ch, 0, vict, TO_CHAR);
		act("$N guts you!", FALSE, vict, 0, ch, TO_CHAR);
		act("$n brutally guts $N!", FALSE, ch, 0, vict, TO_NOTVICT);

		act("$N looks down in horror as their intestines spill out!",
			 FALSE, ch, 0, vict, TO_ROOM);
		act("$N looks down in horror as their intestines spill out!",
			 FALSE, ch, 0, vict, TO_CHAR);
		act("$N looks down in horror as their intestines spill out!",
			 FALSE, vict, 0, ch, TO_CHAR);

		piece = create_obj();

		piece->name = "intestine";
		piece->short_description = "An icky pile of intestines";
		piece->description = "An icky pile of intestines is here - colon and all.";

		GET_OBJ_WEAR(piece).ITEM_WEAR_TAKE = 1;
		piece->item_number = NOTHING;
		IN_ROOM(piece) = NOWHERE;
		GET_OBJ_TYPE(piece) = ITEM_FOOD;
		GET_OBJ_VAL(piece, 0) = 1;   
		GET_OBJ_VAL(piece, 3) = 1;   /* watch what you eat. */
		GET_OBJ_EXTRA(piece).ITEM_NORENT = 1;
		GET_OBJ_EXTRA(piece).ITEM_NOSELL = 1;
		GET_OBJ_EXTRA(piece).ITEM_NODONATE = 1;
		GET_OBJ_WEIGHT(piece) = 1;
		GET_OBJ_RENT(piece) = 1;
		GET_OBJ_TIMER(piece) = max_npc_corpse_time;
		obj_to_room(piece, ch->in_room);

		update_pos(vict);
	}
	WAIT_STATE(ch, PULSE_VIOLENCE * 2);
}


ACMD(do_hurl)
{ 
	struct char_data *vict;
	struct obj_data *obj;
	char buf[MAX_INPUT_LENGTH], buf2[MAX_INPUT_LENGTH];
	int goal = GET_SKILL(ch, SKILL_THROW);
	int roll = rand_number(1, 101);
	int damage_val, destroy = FALSE;

	if (IS_NPC(ch) || !goal) {
		send_to_char(ch, "You can't!\r\n");
		return;
	}
	
	two_arguments(argument, buf, buf2);

	if (!(vict = get_char_room_vis(ch, buf2, NULL))) {
		send_to_char(ch, "Throw what?\r\n");
		return;
	}

	if (!(obj = get_obj_in_list_vis(ch, buf, NULL, ch->carrying))) {
		send_to_char(ch, "Throw what?\r\n");
		return;
	}
  
	if (vict == ch) {
		send_to_char(ch, "That would be funny to see.\r\n");
		return;
	}

	damage_val = (GET_STR(ch) / 2) + (GET_OBJ_WEIGHT(obj) / 2) + GET_LEVEL(ch);

	if (roll > goal) {
		/* miss like a mother fucker. */
      damage(ch, vict, 0, SKILL_THROW);

      act("$N throws $p at you and misses by a long shot.", 
			 FALSE, vict, obj, ch, TO_CHAR);
      act("You throw $p at $n but, miss by a long shot.", 
			 FALSE, vict, obj, ch, TO_VICT);
      act("$N throws $p at $n but, misses by a long shot.", 
			 FALSE, vict, obj, ch, TO_NOTVICT);

		if (rand_number(1, 101) < 20) {
			act("$p shatters upon impact against the ground!", 
				 FALSE, 0, obj, 0, TO_ROOM);
			extract_obj(obj);
		}
		
      return;
	} else {
      if (GET_OBJ_TYPE(obj) == ITEM_SCROLL || GET_OBJ_TYPE(obj) == ITEM_NOTE) {
			act("$N hits you upside the head with $p and exclaims, 'Bad Doggie!'",
				 FALSE, vict, obj, ch, TO_CHAR);
			act("You hit $n in the head with $p and exclaim, 'Bad Doggie!'", 
				 FALSE, vict, obj, ch, TO_VICT);
			act("$N hits $n in the head with $p and exclaims, 'Bad Doggie!'", 
				 FALSE, vict, obj, ch, TO_NOTVICT);

			destroy = FALSE;
			damage_val = 1; /* Should probably be zero damage... */
      } else if (GET_OBJ_TYPE(obj) == ITEM_WEAPON) {
			act("$N throws $p at you and it lands square in your chest.", 
				 FALSE, vict, obj, ch, TO_CHAR);
			act("You throw $p at $n and it lands square in $s chest.", 
				 FALSE, vict, obj, ch, TO_VICT);
			act("$N throws $p at $n and it lands square in $s chest.", 
				 FALSE, vict, obj, ch, TO_NOTVICT);

			if (rand_number(1, 101) < 10)
				destroy = TRUE;

			/* Add in some of the weapon's damage potential! */
			damage_val += (dice(GET_OBJ_VAL(obj, 1), GET_OBJ_VAL(obj, 2)) / 2);
      } else if (GET_OBJ_TYPE(obj) == ITEM_POTION) {
			act("$N throws $p at you and it shatters in your face.",
				 FALSE, vict, obj, ch, TO_CHAR);
			act("You throw $p at $n and it shatters in $m face.", 
				 FALSE, vict, obj, ch, TO_VICT);
			act("$N throws $p at $n and it shatters in $m face.", 
				 FALSE, vict, obj, ch, TO_NOTVICT);
        
			destroy = TRUE;
			
			if (rand_number(1, 100) < 50)
				mag_objectmagic(vict, obj, buf);

			/* Potions ain't gonna do a lot of damage... */
			damage_val /= 2; 
      } else {
			act("$N throws $p and hits you square in the chest.",
				 FALSE, vict, obj, ch, TO_CHAR);
			act("You throw $p at $n and hit $m in the chest.", 
				 FALSE, vict, obj, ch, TO_VICT);
			act("$N throws $p at $n and hits $m in the chest.", 
				 FALSE, vict, obj, ch, TO_NOTVICT);

			if (rand_number(1, 101) < 25)
				destroy = TRUE;
      }
	}

	damage(ch, vict, damage_val, SKILL_THROW);
	WAIT_STATE(ch, PULSE_VIOLENCE * 2);

	if (destroy) {
		act("$p shatters upon impact against $N!", 
			 FALSE, ch, obj, vict, TO_ROOM);
		extract_obj(obj);
	} else {
		obj_from_char(obj);
		obj_to_room(obj, IN_ROOM(vict));
	}
}


ACMD(do_shoot)
{ 
   char arg2[MAX_INPUT_LENGTH];
   char arg1[MAX_INPUT_LENGTH];
   int dir, range;
	struct obj_data *bow = GET_EQ(ch, WEAR_WIELD);
   struct obj_data *missile = GET_EQ(ch, WEAR_HOLD);
	int type;
 
   if (!bow) {
		send_to_char(ch, "You need to wield a missile weapon!\r\n");
		return;
   }
 
   if (!missile) {
		send_to_char(ch, "You need to be holding some ammunition!\r\n");
		return;
   }
 
	type = GET_OBJ_TYPE(bow);
	if (type != ITEM_SLING && type != ITEM_BOW && type != ITEM_CROSSBOW)
		send_to_char(ch, "That's not a missile weapon you're wielding!\r\n");
	
   if ((type == ITEM_SLING    && GET_OBJ_TYPE(missile) == ITEM_ROCK)  ||
		 (type == ITEM_BOW      && GET_OBJ_TYPE(missile) == ITEM_ARROW) ||
		 (type == ITEM_CROSSBOW && GET_OBJ_TYPE(missile) == ITEM_BOLT))
		range = GET_OBJ_VAL(bow, 1);
	else {
		send_to_char(ch, "Your ammunition does not match your weapon!\r\n");
		return;
	}
 
   two_arguments(argument, arg1, arg2);
 
   if (!*arg1 || !*arg2) {
		send_to_char(ch, "You should try: shoot <someone> <direction>\r\n");
		return;
   }
 
	if (IS_DARK(IN_ROOM(ch)) && !CAN_SEE_IN_DARK(ch)) { 
		send_to_char(ch, "You can't see that far.\r\n");
		return;
	}
 
   if ((dir = search_block(arg2, dirs, FALSE)) < 0) {
		send_to_char(ch, "What direction?\r\n");
		return;
   }
 
   if (!CAN_GO(ch, dir)) { 
		send_to_char(ch, "Something blocks the way!\r\n");
		return;
   }
 
   if (range > MAX_MISSILE_RANGE) {
		log("SYSERR: Missile has too great a range! (%s has %d range, %d max)",
			 bow->short_description, range, MAX_MISSILE_RANGE);
		range = MAX_MISSILE_RANGE;
	}
	
   if (range < 1) {
		log("SYSERR: Missile has a range less then one! (%s has %d range)",
			 bow->short_description, range);
		range = 1;
	}
	
   fire_missile(ch, arg1, missile, WEAR_HOLD, range, dir);
}
 
 
ACMD(do_throw)
{
	/* sample format: throw monkey east
	 *
	 * This would throw a throwable or grenade object wielded into the
	 * room 1 east of the pc's current room. The chance to hit the
	 * monkey would be calculated based on the pc's skill.  if the
	 * wielded object is a grenade then it does not 'hit' for damage,
	 * it is merely dropped into the room. (the timer is set with the
	 * 'pull pin' command.) */
 
   struct obj_data *missile;
   int dir, range = 1, type;
   char arg2[MAX_INPUT_LENGTH];
   char arg1[MAX_INPUT_LENGTH];

   two_arguments(argument, arg1, arg2);
 
	/* only two types of throwing objects: 
		ITEM_THROW - knives, stones, etc
		ITEM_GRENADE - calls tick_grenades.c . */
 
   missile = GET_EQ(ch, WEAR_WIELD);
   if (!missile) {
		send_to_char(ch, "You should wield a throwing weapon first!\r\n");
		return;
   }
   
	type = GET_OBJ_TYPE(missile);
   if (type != ITEM_THROW && type != ITEM_GRENADE) {
		send_to_char(ch, "That's not a throwing weapon you're wielding!\r\n");
		return;
   }
   
   if (type == ITEM_GRENADE) {
		if (!*arg1) {
			send_to_char(ch, "You should try: throw <direction>\r\n");
			return;
		}
		if ((dir = search_block(arg1, dirs, FALSE)) < 0) { 
			send_to_char(ch, "What direction?\r\n");
			return;
		}
   } else { /* if (type == ITEM_THROW) */
		two_arguments(argument, arg1, arg2);
 
		if (!*arg1 || !*arg2) {
			send_to_char(ch, "You should try: throw <someone> <direction>\r\n");
			return;
		}
 
		/* arg2 must be a direction */
		if ((dir = search_block(arg2, dirs, FALSE)) < 0) { 
			send_to_char(ch, "What direction?\r\n");
			return;
		}
   }
 
	/* make sure we can go in the direction throwing. */
   if (!CAN_GO(ch, dir)) { 
		send_to_char(ch, "Something blocks the way!\r\n");
		return;
   }
 
   fire_missile(ch, arg1, missile, WEAR_WIELD, range, dir);
}   


ACMD(do_draw)
{
	struct obj_data *obj;
	char arg[MAX_INPUT_LENGTH];
	
   one_argument(argument, arg);
 
   if (!*arg) {
		send_to_char(ch, "Pull what?\r\n");
		return;
   }

	/* For now only pull pin will work and must be wielding a
	 * grenade. */
   if (!str_cmp(arg, "pin")) {
		obj = GET_EQ(ch, WEAR_WIELD);
		
		if (obj) {
			if (GET_OBJ_TYPE(obj) == ITEM_GRENADE) {
				send_to_char(ch, "You pull the pin, the grenade is now live!\r\n");
				GET_OBJ_EXTRA(obj).ITEM_LIVE_GRENADE = 1;
			} else
				send_to_char(ch, "That's NOT a grenade!\r\n");
		} else
			send_to_char(ch, "You aren't wielding anything!\r\n");
   }
 
	/* put other 'pull' options here later */
}

