/* ************************************************************************
 *   File: mobact.c                                      Part of CircleMUD *
 *  Usage: Functions for generating intelligent (?) behavior in mobiles    *
 *                                                                         *
 *  All rights reserved.  See license.doc for complete information.        *
 *                                                                         *
 *  Copyright (C) 1993, 94 by the Trustees of the Johns Hopkins University *
 *  CircleMUD is based on DikuMUD, Copyright (C) 1990, 1991.               *
 ************************************************************************ */

#include "conf.h"
#include "sysdep.h"


#include "structs.h"
#include "utils.h"

#include "comm.h"
#include "constants.h"
#include "db.h"
#include "handler.h"
#include "interpreter.h"
#include "mobprog.h"
#include "spells.h"


/* external globals */
extern int no_specials;

/* external functions */
void perform_wear(struct char_data * ch, struct obj_data * obj, int where);
void perform_remove(struct char_data * ch, int pos);
int invalid_class(struct char_data *ch, struct obj_data *obj);
int apply_ac(struct char_data *ch, int eq_pos);
bool can_wear(struct obj_data *obj, int part);
SPECIAL(shop_keeper);
ACMD(do_get);
ACMD(do_action);

/* local functions */
void mobile_activity(void);
void clearMemory(struct char_data *ch);
bool aggressive_mob_on_a_leash(struct char_data *slave, struct char_data *master, struct char_data *attack);
void mob_wield(struct char_data *ch);
void mob_wear_armor(struct char_data *ch, int wear_pos);

void mobile_activity(void)
{
	struct char_data *ch, *next_ch, *vict;
	struct obj_data *obj, *best_obj;
	int door, found, max;
	memory_rec *names;

	for (ch = character_list; ch; ch = next_ch) {
		next_ch = ch->next;

		if (!IS_MOB(ch))
			continue;

		/* Examine call for special procedure */
		if (MOB_FLAGGED(ch, MOB_SPEC) && !no_specials) {
			if (mob_index[GET_MOB_RNUM(ch)].func == NULL) {
				log("SYSERR: %s (#%d): Attempting to call non-existing mob function.",
					 GET_NAME(ch), GET_MOB_VNUM(ch));
				MOB_FLAGS(ch).MOB_SPEC = 0;
			} else {
				char actbuf[MAX_INPUT_LENGTH] = "";
				if ((mob_index[GET_MOB_RNUM(ch)].func) (ch, ch, 0, actbuf))
					continue;		/* go to next char */
			}
		}

		if (!is_empty(world[IN_ROOM(ch)].zone))
			mprog_random_trigger(ch);

		/* If the mob has no specproc, do the default actions */
		if (FIGHTING(ch) || !AWAKE(ch))
			continue;

		/* Scavenger (picking up objects) */
		if (MOB_FLAGGED(ch, MOB_SCAVENGER)) {
			if (world[IN_ROOM(ch)].contents && !rand_number(0, 10)) {
				max = 1;
				best_obj = NULL;
				for (obj = world[IN_ROOM(ch)].contents; obj; obj = obj->next_content)
					if (CAN_GET_OBJ(ch, obj) && GET_OBJ_COST(obj) > max) {
						best_obj = obj;
						max = GET_OBJ_COST(obj);
					}
				if (best_obj != NULL) {
					obj_from_room(best_obj);
					obj_to_char(best_obj, ch);
					act("$n gets $p.", FALSE, ch, best_obj, 0, TO_ROOM);
				}
			}
		}

		/* Mob Movement */
		if (IN_ROOM(ch) != NOWHERE) {
			if (!MOB_FLAGGED(ch, MOB_SENTINEL) && (GET_POS(ch) == POS_STANDING) &&
				 ((door = rand_number(0, 18)) < NUM_OF_DIRS) && CAN_GO(ch, door) &&
				 (!ROOM_FLAGGED(EXIT(ch, door)->to_room, ROOM_NOMOB) &&
				  !ROOM_FLAGGED(EXIT(ch, door)->to_room, ROOM_DEATH)) &&
				 (!MOB_FLAGGED(ch, MOB_STAY_ZONE) ||
				  (world[EXIT(ch, door)->to_room].zone == world[IN_ROOM(ch)].zone))) {
				perform_move(ch, door, 1);
			}
		} else
			log("In mobile_activity(), %s is in room NOWHERE!\r\n", GET_NAME(ch));

 	   /* MOB Prog foo */
		if (IS_NPC(ch) && ch->mpactnum > 0 && 
			 !is_empty(world[IN_ROOM(ch)].zone)) {

			MPROG_ACT_LIST *tmp_act, *tmp2_act;
			for (tmp_act = ch->mpact; tmp_act; tmp_act = tmp_act->next) {
				mprog_wordlist_check(tmp_act->buf, ch, tmp_act->ch,
											tmp_act->obj, tmp_act->vo, ACT_PROG);
				free(tmp_act->buf);
			}
			for (tmp_act = ch->mpact; tmp_act; tmp_act = tmp2_act) {
				tmp2_act = tmp_act->next;
				free(tmp_act);
			}
			ch->mpactnum = 0;
			ch->mpact = NULL;
		}

		/* Aggressive Mobs */
		if (MOB_FLAGGED(ch, MOB_AGGRESSIVE) ||
			 MOB_FLAGGED(ch, MOB_AGGR_EVIL) ||
			 MOB_FLAGGED(ch, MOB_AGGR_NEUTRAL) ||
			 MOB_FLAGGED(ch, MOB_AGGR_GOOD)) {
			found = FALSE;
			for (vict = world[IN_ROOM(ch)].people; vict && !found; vict = vict->next_in_room) {

				/* if (IS_NPC(vict) || !CAN_SEE(ch, vict) || PRF_FLAGGED(vict, PRF_NOHASSLE)) */
				/* Let aggressive mobs fight other mobs, not just PCs...
				 * If you don't like this, comment out the line below and
				 * uncomment the line above.  -ejg  */
				if (!CAN_SEE(ch, vict) || (!IS_NPC(vict) && PRF_FLAGGED(vict, PRF_NOHASSLE))) 
					continue;

				if (vict == ch)
					continue;

				if (MOB_FLAGGED(ch, MOB_WIMPY) && AWAKE(vict))
					continue;

				if (MOB_FLAGGED(ch, MOB_AGGRESSIVE  ) ||
					 (MOB_FLAGGED(ch, MOB_AGGR_EVIL   ) && IS_EVIL(vict)) ||
					 (MOB_FLAGGED(ch, MOB_AGGR_NEUTRAL) && IS_NEUTRAL(vict)) ||
					 (MOB_FLAGGED(ch, MOB_AGGR_GOOD   ) && IS_GOOD(vict))) {

					/* Can a master successfully control the charmed monster? */
					if (aggressive_mob_on_a_leash(ch, ch->master, vict))
						continue;

					hit(ch, vict, TYPE_UNDEFINED);
					found = TRUE;
				}
			}
		}

		/* Mob Memory */
		if (MOB_FLAGGED(ch, MOB_MEMORY) && MEMORY(ch)) {
			found = FALSE;
			for (vict = world[IN_ROOM(ch)].people; vict && !found; vict = vict->next_in_room) {
				if (IS_NPC(vict) || !CAN_SEE(ch, vict) || PRF_FLAGGED(vict, PRF_NOHASSLE))
					continue;

				for (names = MEMORY(ch); names && !found; names = names->next) {
					if (names->id != GET_IDNUM(vict))
						continue;

					/* Can a master successfully control the charmed monster? */
					if (aggressive_mob_on_a_leash(ch, ch->master, vict))
						continue;

					found = TRUE;
					act("'Hey!  You're the fiend that attacked me!!!', exclaims "
						 "$n.", FALSE, ch, 0, 0, TO_ROOM);
					hit(ch, vict, TYPE_UNDEFINED);
				}
			}
		}

		/*
		 * Charmed Mob Rebellion
		 *
		 * In order to rebel, there need to be more charmed monsters
		 * than the person can feasibly control at a time.  Then the
		 * mobiles have a chance based on the charisma of their leader.
		 *
		 */
		if (AFF_FLAGGED(ch, AFF_CHARM) && ch->master && 
			 num_followers_charmed(ch->master) > cha_app[(int)GET_CHA(ch->master)].num_followers) {
			if (!aggressive_mob_on_a_leash(ch, ch->master, ch->master)) {
				if (CAN_SEE(ch, ch->master) && !PRF_FLAGGED(ch->master, PRF_NOHASSLE))
					hit(ch, ch->master, TYPE_UNDEFINED);
				stop_follower(ch);
			}
		}

		/* Helper Mobs */
		if (MOB_FLAGGED(ch, MOB_HELPER) && 
			 !AFF_FLAGGED(ch, AFF_BLIND) && !AFF_FLAGGED(ch, AFF_CHARM)) {
			found = FALSE;
			for (vict = world[IN_ROOM(ch)].people; vict && !found; vict = vict->next_in_room) {
				if (ch == vict || !IS_NPC(vict) || !FIGHTING(vict))
					continue;
				if (IS_NPC(FIGHTING(vict)) || ch == FIGHTING(vict))
					continue;

				act("$n jumps to the aid of $N!", FALSE, ch, 0, vict, TO_ROOM);
				hit(ch, FIGHTING(vict), TYPE_UNDEFINED);
				found = TRUE;
			}
		}

		/* Add new mobile actions here */
		if (ch->mob_specials.newitem) {
			mob_wield(ch);

			mob_wear_armor(ch, WEAR_BODY);
			mob_wear_armor(ch, WEAR_HEAD);
			mob_wear_armor(ch, WEAR_LEGS);
			mob_wear_armor(ch, WEAR_FEET);
			mob_wear_armor(ch, WEAR_HANDS);
			mob_wear_armor(ch, WEAR_ARMS);
			mob_wear_armor(ch, WEAR_SHIELD);
			mob_wear_armor(ch, WEAR_WAIST);
			ch->mob_specials.newitem = 0;
		}
	}				/* end for() */
}



/* Mob Memory Routines */

/* make ch remember victim */
void remember(struct char_data *ch, struct char_data *victim)
{
	memory_rec *tmp;
	bool present = FALSE;

	if (!IS_NPC(ch) || IS_NPC(victim) || PRF_FLAGGED(victim, PRF_NOHASSLE))
		return;

	for (tmp = MEMORY(ch); tmp && !present; tmp = tmp->next)
		if (tmp->id == GET_IDNUM(victim))
			present = TRUE;

	if (!present) {
		CREATE(tmp, memory_rec, 1);
		tmp->next = MEMORY(ch);
		tmp->id = GET_IDNUM(victim);
		MEMORY(ch) = tmp;
	}
}


/* make ch forget victim */
void forget(struct char_data *ch, struct char_data *victim)
{
	memory_rec *curr, *prev = NULL;

	if (!(curr = MEMORY(ch)))
		return;

	while (curr && curr->id != GET_IDNUM(victim)) {
		prev = curr;
		curr = curr->next;
	}

	if (!curr)
		return;			/* person wasn't there at all. */

	if (curr == MEMORY(ch))
		MEMORY(ch) = curr->next;
	else
		prev->next = curr->next;

	free(curr);
}


/* erase ch's memory */
void clearMemory(struct char_data *ch)
{
	memory_rec *curr, *next;

	curr = MEMORY(ch);

	while (curr) {
		next = curr->next;
		free(curr);
		curr = next;
	}

	MEMORY(ch) = NULL;
}


/*
 * An aggressive mobile wants to attack something.  If
 * they're under the influence of mind altering PC, then
 * see if their master can talk them out of it, eye them
 * down, or otherwise intimidate the slave.
 */
bool aggressive_mob_on_a_leash(struct char_data *slave, struct char_data *master, struct char_data *attack)
{
	static int snarl_cmd;
	int dieroll;

	if (!master || !AFF_FLAGGED(slave, AFF_CHARM))
		return (FALSE);

	if (!snarl_cmd)
		snarl_cmd = find_command("snarl");

	/* Sit. Down boy! HEEEEeeeel! */
	dieroll = rand_number(1, 20);
	if (dieroll != 1 && 
		 (dieroll == 20 || 
		  dieroll > 10 - cha_app[(int)GET_CHA(master)].loyalty + cha_app[(int)GET_CHA(slave)].reaction)) {
		if (snarl_cmd > 0 && attack && !rand_number(0, 3)) {
			char victbuf[MAX_NAME_LENGTH + 1];

			strncpy(victbuf, GET_NAME(attack), sizeof(victbuf));	/* strncpy: OK */
			victbuf[sizeof(victbuf) - 1] = '\0';

			do_action(slave, victbuf, snarl_cmd, 0);
		}

		/* Success! But for how long? Hehe. */
		return (TRUE);
	}

	/* So sorry, now you're a player killer... Tsk tsk. */
	return (FALSE);
}


void mob_wield(struct char_data *ch)
{
	struct obj_data *wield_obj, *best_obj, *obj;
	int i, a1 = 0, a2 = 0;
	float max, dam2; 
	
	if (mob_index[GET_MOB_RNUM(ch)].func != shop_keeper) {
		if (GET_EQ(ch, WEAR_WIELD)) {
			wield_obj = ch->equipment[WEAR_WIELD];
			best_obj = wield_obj;

			for (i = 0, a1 = 0; i < MAX_OBJ_AFFECT; i++) {
				if (wield_obj->affected[i].location == APPLY_DAMROLL)
					a1 += wield_obj->affected[i].modifier;
			}

			max = (GET_OBJ_VAL(wield_obj, 1) * GET_OBJ_VAL(wield_obj, 2)) + a1;
      } else {
			wield_obj = NULL;
			best_obj = NULL;
			max = 1;
		}

		for (obj = ch->carrying; obj; obj = obj->next_content) {
			if (GET_OBJ_TYPE(obj) == ITEM_WEAPON) {
				for (i = 0, a2 = 0; i < MAX_OBJ_AFFECT; i++) {
					if (obj->affected[i].location == APPLY_DAMROLL)
						a2 += obj->affected[i].modifier;
				}

				dam2 = (GET_OBJ_VAL(obj, 1) * GET_OBJ_VAL(obj, 2)) + a2;

				if (dam2 > max) {
					if (!invalid_align(ch, obj) && !invalid_class(ch, obj)) {
						best_obj = obj;

						for (i = 0, a2 = 0; i < MAX_OBJ_AFFECT; i++) {
							if (best_obj->affected[i].location == APPLY_DAMROLL)
								a2 += best_obj->affected[i].modifier;
						}

						max = (GET_OBJ_VAL(best_obj, 1) * GET_OBJ_VAL(best_obj, 2)) + a2;
					}
				}
			}
		}
		if (GET_EQ(ch, WEAR_WIELD)) {
			if (wield_obj != best_obj) {
				perform_remove(ch, WEAR_WIELD);
				perform_wear(ch, best_obj, WEAR_WIELD);
			}
		} else {
			if (best_obj)
				perform_wear(ch, best_obj, WEAR_WIELD);
		}
	}
}


int potential_apply_ac(struct obj_data *obj, int eq_pos)
{
	int factor;

	if (!obj) {
		core_dump();
		return (0);
	}

	if (GET_OBJ_TYPE(obj) != ITEM_ARMOR)
		return (0);

	switch (eq_pos) {

		case WEAR_BODY:
			factor = 3;
			break;			/* 30% */
		case WEAR_HEAD:
			factor = 2;
			break;			/* 20% */
		case WEAR_LEGS:
			factor = 2;
			break;			/* 20% */
		default:
			factor = 1;
			break;			/* all others 10% */
	}

	return (factor * GET_OBJ_VAL(obj, 0));
}

void mob_wear_armor(struct char_data *ch, int wear_pos)
{
	struct obj_data *wear_obj, *best_obj, *obj;
	int i, a1 = 0, a2 = 0;
	float max, dam2; 
	
	if (mob_index[GET_MOB_RNUM(ch)].func != shop_keeper) {
		
		if (GET_EQ(ch, wear_pos)) {
			wear_obj = ch->equipment[wear_pos];
			best_obj = wear_obj;
			
			if (GET_OBJ_TYPE(wear_obj) == ITEM_ARMOR) {
				for (i = 0, a1 = 0; i < MAX_OBJ_AFFECT; i++) {
					if (wear_obj->affected[i].location == APPLY_AC)
						a1 += wear_obj->affected[i].modifier;
				}
				max = apply_ac(ch, wear_pos);
			}
      } else {
			wear_obj = NULL;
			best_obj = NULL;
			max = 1;
		}
		
		for (obj = ch->carrying; obj; obj = obj->next_content) {
			if (GET_OBJ_TYPE(obj) == ITEM_ARMOR && can_wear(obj, wear_pos)) {
				for (i = 0, a2 = 0; i < MAX_OBJ_AFFECT; i++) {
					if (obj->affected[i].location == APPLY_AC)
						a2 += obj->affected[i].modifier;
				}

				dam2 = potential_apply_ac(obj, wear_pos) + a2;

				if (dam2 > max) {
					if (!invalid_align(ch, obj) && !invalid_class(ch, obj)) {
						best_obj = obj;

						for (i = 0, a2 = 0; i < MAX_OBJ_AFFECT; i++) {
							if (best_obj->affected[i].location == APPLY_AC)
								a2 += best_obj->affected[i].modifier;
						}

						max = potential_apply_ac(best_obj, wear_pos) + a2;
					}
				}
			}
		}
		if (GET_EQ(ch, wear_pos)) {
			if (wear_obj != best_obj) {
				perform_remove(ch, wear_pos);
				perform_wear(ch, best_obj, wear_pos);
			}
		} else {
			if (best_obj)
				perform_wear(ch, best_obj, wear_pos);
		}
	}
}
